/**************************************************************/
/*                                                            */
/*         Modul zum Parsen von Funktionsstrings              */
/*                                                            */
/* 11.09.91 Siegfried Goetz:                                  */
/*           Implementation durch Umsetzung von Modula-Modul  */
/*           aus dem Buch "MODULA-2",S.Duerholt,J.Schnur,     */
/*           Markt&Technik in Turbo-C                         */
/*                                                            */
/* 26.10.93 Siegfied Goetz:                                   */
/*           Erweiterung der Differentiationsmoeglichkeiten   */
/*           auf alle Variablen.                              */
/*           Verbesserung der Fehlermeldungsausgabe           */
/*                                                            
   25.3.95 Siegfried Goetz:
             Es koennen jetzt auch mehrdimensionale Funktionen
	     gelernt werden incl. die Dazugehoerigen Ableitungen
             zu jedem Parameter.
***************************************************************/

#include  <string.h>
#include  <ctype.h>
#include  <stdlib.h>
#include  <stddef.h>
#include  <stdio.h> 
#include  <math.h>

#define BEZHIGH  16
#define MAXHIGH  4096
#define VARMAX   10

/* typen */
    
typedef char BEZSTRING[BEZHIGH];
typedef char MAXSTRING[MAXHIGH];      

typedef double  FUNR1(double); /* Reelle, eindimensionale Funktion */
typedef double  FUNR2(double,double); 
typedef double  FUNR3(double,double,double); 
typedef double  FUNR4(double,double,double,double); 
typedef double  FUNR5(double,double,double,double,double); 
typedef double  FUNR6(double,double,double,double,double,double); 
typedef double  FUNR7(double,double,double,double,double,double,double); 
typedef double  FUNR8(double,double,double,double,double,double,double,double); 
typedef double  FUNR9(double,double,double,double,double,double,double,double,double);
typedef double  FUNR10(double,double,double,double,double,double,double,double,double,double);

typedef enum
        { false,
          true
        }BOOLEAN;
typedef enum
        {
          errOK,         /* Kein Fehler */
          errCharacter,  /* unerlaubtes Zeichen im Text */
          errBezeichner, /* undeklarierter Bezeichner */
          errKlammerAuf, /* "(" erwartet */
          errKlammerZu,  /* ")" erwartet */
          errKomma,      /* "," erwartet */
          errAusdruck,   /* Arithmetischer Ausdruck erwartet */
          errOperator    /* Operator erwartet */
        } ERRORTYPE;

typedef enum  
        {
          SymUnbekannt, SymEnde,
          KlammerAuf, KlammerZu,Komma,
          OpMinus, OpPlus, OpDurch, OpMal, OpHoch, OpNeg,
          SymFunR1, SymVarRR, SymKoRR, SymPnode
        } SYMTYPE;

        
typedef struct PARSERNODE  *PARSERPTR;
typedef struct BEZEICHNER  *BEZPTR;

struct BEZEICHNER
        {
          BEZPTR  next;
          BEZSTRING Name;
          SYMTYPE BezArt;
          union 
          { 
	    union{
	      struct{
		FUNR1 *f   ;     /* wenn bezart=SymFunR1 */
		int va    ;      /* Anzahl der Parameter */
	      } fkt;
	      struct{ FUNR2 *f2;int va    ;} fkt2;   
	      struct{ FUNR3 *f3;int va    ;} fkt3;   
	      struct{ FUNR4 *f4;int va    ;} fkt4;   
	      struct{ FUNR5 *f5;int va    ;} fkt5;   
	      struct{ FUNR6 *f6;int va    ;} fkt6;   
	      struct{ FUNR7 *f7;int va    ;} fkt7;   
	      struct{ FUNR8 *f8;int va    ;} fkt8;   
	      struct{ FUNR9 *f9;int va    ;} fkt9;   
	      struct{ FUNR10 *f10;int va    ;} fkt10;   
	      
	    } dim;
            double value;    /* wenn bezart=SymVarRR */
	    PARSERPTR pnode; /* wenn bezart=SymPnode */
          } art;
        };

        struct PARSERNODE
        {
          SYMTYPE OperArt;
          union
          {
            PARSERPTR Operand;
            double KoRR;
            struct { PARSERPTR nr1,nr2;} oper;
            struct { BEZPTR    BezFktl;
                     PARSERPTR Parameter[VARMAX];
                   } funk;
            BEZPTR Variable;
          } ops;
        };
      
      /* definitionen fuer den differenzierer */
      
typedef   struct DIFFER *DIFFERPTR;
struct DIFFER
    {
         DIFFERPTR  next;
         BEZPTR     Funktion;
         PARSERPTR  Ableitung[VARMAX];
    };
    
DIFFERPTR ListeDerAbleitungen=NULL;
BEZPTR    LnFunktion=NULL;   /* fuers Ableiten von a^b */
BEZPTR VarX,SymX;
         
     
/* globale Variablen */                
MAXSTRING  ScanZeile;
unsigned int ScanPosition, ScanErrPos;
BOOLEAN  SyntaxError=false, ArithmeticError=false, DifferError=false;
ERRORTYPE ErrorArt;
BEZPTR  BezeichnerListe=NULL;      /* Liste aller Bezeichner (Funktionen) */
struct PARSERNODE  ScanNode;       /* naechste Symbol vom Scanner  */
char ScanChar;

/* Funktionsprototypen */

PARSERPTR makeTerm(void);
void Optimiere(PARSERPTR *f);
void PrinTree(PARSERPTR p);
PARSERPTR parse( char *EingabeZeile);



void error(ERRORTYPE e)
{
    if (e == errOK)
    {
        SyntaxError = false;
        ErrorArt = errOK;
    }    
    else
    {
        if (SyntaxError==false)
        {
            ErrorArt = e;
            SyntaxError = true;
        }    
    }
}



BEZPTR HoleBezeichner(BEZSTRING name)
/* ....................................................................
 * liefert zu einem BezeichnerNamen einen Pointer auf den Bezeichner-
 * deskriptor, wenn er in der Bezeichnerliste steht, sonst NULL
 */
{   BEZPTR p;
    p = BezeichnerListe;
    while ((p!= NULL) && (strcasecmp((p->Name),name) != 0))  p = p->next;
    
    return p;
}


void LerneFunktion( BEZSTRING name,const void *funk,int vanz)
{
    BEZPTR neuer;
    if (HoleBezeichner(name) != NULL){         /* schon definiert! */
        fprintf(stderr,"Funktion %s ist bereits definiert\n",name);
        exit(-100);
    }
    neuer = malloc(sizeof(struct BEZEICHNER)); /* Platz holen ... */
    neuer->next = BezeichnerListe;             /* fuer Verkettung */
    strcpy(neuer->Name,name);                  /* belegen... */
    neuer->BezArt = SymFunR1;
    neuer->art.dim.fkt.f= (FUNR1*)funk;             /* Pointer auf Funktion */
    neuer->art.dim.fkt.va=vanz ;                   /* Parameteranzahl */
    BezeichnerListe = neuer;                   /* in die Liste einbauen */
}

void LernePFunktion( BEZSTRING name,char *funk)
{
    BEZPTR neuer;
    if (HoleBezeichner(name) != NULL){         /* schon definiert! */
        fprintf(stderr,"Funktion %s ist bereits definiert\n",name);
        exit(-100);
    }
    neuer = malloc(sizeof(struct BEZEICHNER)); /* Platz holen ... */
    neuer->next = BezeichnerListe;             /* fuer Verkettung */
    strcpy(neuer->Name,name);                  /* belegen... */
    neuer->BezArt = SymPnode;
    neuer->art.pnode = parse(funk);            /* Funktion Parsen */
    BezeichnerListe = neuer;                   /* in die Liste einbauen */
}

void LerneVariable(BEZSTRING name, double wert)
{
    BEZPTR neuer;
    if (HoleBezeichner(name) != NULL){         /* schon definiert! */
            fprintf(stderr,"Variable %s ist bereits definiert\n",name);
            exit(-101);
    }
    neuer = malloc(sizeof(struct BEZEICHNER)); /* Platz holen ... */
    neuer->next = BezeichnerListe;             /* Belegen ... */
    strcpy(neuer->Name,name);
    neuer->BezArt = SymVarRR;
    neuer->art.value = wert;
    BezeichnerListe = neuer;                   /* ... in die Liste einbauen */
}

void SetzeVariable(BEZPTR bez, double wert)
{
    if (bez == NULL){
        fprintf(stderr,"Variablenpointer ist NULL\n");
        exit(-102);
    }
    if (bez->BezArt != SymVarRR){               /* muss Variable sein */
        fprintf(stderr,"Nur Variablen kann ein Wert zugewiesen werden\n");
        exit(-103);
    }
    bez->art.value = wert;
}

void ClearNode(PARSERPTR node)
/* loescht einen ParserKnoten. Rein prophylaktische Angelegenheit */
{
  int i;
  switch (node->OperArt)
    {
    case OpMinus :
    case OpPlus :
    case OpMal :
    case OpDurch :
    case OpHoch : 
      node->ops.oper.nr1=NULL;
      node->ops.oper.nr2=NULL;
      break;
    case OpNeg :
      node->ops.Operand=NULL;break;
    case SymFunR1 :
      for(i=0;i<VARMAX;i++) node->ops.funk.Parameter[i] = NULL;
      node->ops.funk.BezFktl = NULL; break;
    case SymVarRR :
      node->ops.Variable = NULL; break;
    }                   
}

/*******************************************************************/
/******************** Modul Formel zu String ***********************/ 
/*******************************************************************/

/* um einen Baum in einen Formelstring umzuwandeln muss die Funktion
   ***FormelzuString*** aufgerufen werden. Als Parameter werden der
   Funktionenbaumpointer Uebergeben und ein Zeiger auf einen genuegend
   grossen String, da ein Stringueberlauf nicht kontroliert wird. */
   
char *formel;

    void schreib(char *s)
    {
        while ( *s != 0) *formel++ = *s++;
    }
    
    void wandle(PARSERPTR p,SYMTYPE level)
    {
      int i;
        char konst[20];
        if ((p->OperArt) < level) schreib("(");
        switch  (p->OperArt)
        {
            case OpNeg:
                schreib("-");
                wandle(p->ops.Operand, OpNeg);break;
            case OpMinus:
                wandle(p->ops.oper.nr1,OpMinus);
                schreib("-");
                wandle(p->ops.oper.nr2,OpDurch);break;
            case OpPlus:
                wandle(p->ops.oper.nr1, OpPlus);
                schreib("+");
                wandle(p->ops.oper.nr2, OpDurch);break;
            case OpDurch:
                wandle(p->ops.oper.nr1, OpDurch);
                schreib("/");
                wandle(p->ops.oper.nr2, OpHoch);break;
            case OpMal:
                wandle(p->ops.oper.nr1, OpMal);
                schreib("*");
                wandle(p->ops.oper.nr2, OpHoch);break;
            case OpHoch:
                wandle(p->ops.oper.nr1, OpNeg);
                schreib("^");
                wandle(p->ops.oper.nr2, OpHoch);break;
            case SymVarRR:
                schreib(p->ops.Variable->Name);break;
/*            case SymFunR1:
                schreib(p->ops.funk.BezFktl->Name);
		schreib("(");
		for(i=0;i<p->ops.funk.BezFktl->art.dim.fkt.va;i++){
		    wandle(p->ops.funk.Parameter[i],SymKoRR);
		    if(i < p->ops.funk.BezFktl->art.dim.fkt.va-1) schreib(",");
		}
		schreib(")");
		break; */
            case SymFunR1:
                schreib(p->ops.funk.BezFktl->Name);
		if(p->ops.funk.BezFktl->art.dim.fkt.va>1) schreib("(");
		for(i=0;i<p->ops.funk.BezFktl->art.dim.fkt.va;i++){
		    if(p->ops.funk.BezFktl->art.dim.fkt.va ==1) 
		      wandle(p->ops.funk.Parameter[i],SymKoRR);
		    else
		      wandle(p->ops.funk.Parameter[i],OpNeg);
		    if(i < p->ops.funk.BezFktl->art.dim.fkt.va-1) schreib(",");
		}
		if(p->ops.funk.BezFktl->art.dim.fkt.va>1) schreib(")");
		break;
            case SymKoRR:
                sprintf(konst,"%lg",p->ops.KoRR);
                schreib(konst);
        }
        if (p->OperArt < level) schreib(")");
    }


void BaumZuString(PARSERPTR baum, char *formelstr)
/************************************************/
{
    formelstr[0]=' ';
    if(baum != NULL){
        formel=formelstr;
        wandle(baum, SymEnde);
        *formel = 0;
    }
}

/******************** Modul Formel zu String Ende ******************/ 
/*******************************************************************/

/* -------------------------------------------------------------------
:: nur zu Testzwecken: Ausgabe des kompletten ParserBaumes
\\ ...................................................................
*/
unsigned int i,tiefe;

void printree1 (PARSERPTR p)
  {
    int k;
    printf("\n");
    for (i = 1; i<=tiefe;i++) printf("..|");
    printf("--");
    if (p == NULL) 
        printf("*** NIL ***");
    else
    {
        tiefe++;
        switch (p->OperArt)
        {
         case OpNeg:
                printf("(- (Neg))");
                printree1(p->ops.Operand);break;
         case OpPlus:
         case OpMinus:
         case OpMal  :
         case OpDurch:
         case OpHoch:
                switch (p->OperArt)
                {
                 case OpPlus: printf("[+]");break;
                 case OpMinus: printf("[-]");break;
                 case OpMal: printf("[*]");break;
                 case OpDurch: printf("[/]");break;
                 case OpHoch: printf("[HOCH]");break;
                }
                printree1(p->ops.oper.nr1);
                printree1(p->ops.oper.nr2);break;
         case SymFunR1:
                printf(p->ops.funk.BezFktl->Name); printf("(.)");
		for(k=0;k<p->ops.funk.BezFktl->art.dim.fkt.va;k++)
		  printree1(p->ops.funk.Parameter[k]);break;
         case SymVarRR:
                printf(p->ops.Variable->Name); printf(" = ");
                printf("%lg",(p->ops.Variable->art.value));break;
         case SymKoRR:
                printf("%lg",(p->ops.KoRR));break;
        }
        tiefe--;
    }
  }
  
void PrinTree(PARSERPTR baum)
{
    tiefe = 0;
    printree1(baum);
    printf("\n");
}



/* ====================================================================
 * ==================   1. Teil: der Scanner   ========================
 */




char LiesZeichen( void )
{
    char z;
    unsigned int kkk=MAXHIGH;
    z = ScanChar;
    if( ( ScanPosition <= kkk )   && (ScanZeile[ScanPosition] != 0) )
    {
        ScanPosition++;
        ScanChar = ScanZeile[ScanPosition];
    }    
    else
        ScanChar = 0;     /* Zeile zu Ende */
    return z;
}

void SkipZeichen(void)
{
   LiesZeichen();
}


void LiesWort(char *s)
/* Liest genau ein Wort aus der 'ScanZeile' nach 's' */
{
    do
    {
        *s++ = LiesZeichen();
    } while (isalnum(ScanChar));
    *s = 0;
}


void LiesZahl(void)
/* einlesen einer double- Konstante */
{
    double r,d10,dd=0.0,fak=1.0;
    r = 0.0;
    while (isdigit(ScanChar))
       r = 10*r + LiesZeichen() - '0';
    if (ScanChar == '.')  /* es kommen noch Nachkommastellen... */
    {   SkipZeichen();
        d10 = 0.1;
        while (isdigit(ScanChar)) 
        {
            r += (LiesZeichen()-'0') * d10;
            d10 *= 0.1;
         };   
    };
    if((ScanChar == 'e')||(ScanChar == 'E')){
     	SkipZeichen();
     	if(ScanChar=='-'){
     		SkipZeichen();
     		fak= -1.0;
    	};
     	if(ScanChar=='+'){
     		SkipZeichen();
     		fak=1.0;
    	};
     	while (isdigit(ScanChar))
       		dd = 10*dd + LiesZeichen() - '0';
     	dd=dd*fak;
     	r=r*pow(10.0,dd);
    };


    ScanNode.OperArt = SymKoRR;
    ScanNode.ops.KoRR = r;
}


void LiesSymbol(void)
/*
 * Liest aus der EingabeZeile ein Symbol und traegt es in 'ScanNode' ein
 */
{ 
    BEZSTRING BezName;
    struct BEZEICHNER *bezei;
    if (SyntaxError){           /* Parser nicht richtig gestoppt */
        fprintf(stderr,"Parser nicht richtig gestoppt wegen Syntaxerror\n");
        exit(-104);
    }
    while (ScanChar == ' ') SkipZeichen();
    ScanErrPos = ScanPosition; /* Position merken, falls Fehler auftritt */

    if (isalpha(ScanChar))            /* -------- Bezeichner -------- */
    {    
        LiesWort(BezName);
        bezei = HoleBezeichner(BezName);
        if (bezei == NULL)
        {   ScanNode.OperArt = SymUnbekannt;
            error(errBezeichner);
        }    
        else
            ScanNode.OperArt = bezei->BezArt;
        ClearNode(&ScanNode);
        switch (ScanNode.OperArt)
        {
          case SymFunR1:
                ScanNode.ops.funk.BezFktl = bezei;break;
          case SymVarRR:
                ScanNode.ops.Variable = bezei;break;
          case SymUnbekannt: /* nix */;
        }
    }    
    else if (isdigit(ScanChar))            /* -------- Konstante ---------- */
        LiesZahl();
    else
    {                      /* ---- sonst: Operator oder Satzzeichen ---- */
        switch (ScanChar)
        {
           case '(':    ScanNode.OperArt = KlammerAuf;break;
           case ')':    ScanNode.OperArt = KlammerZu;break;
           case '-':    ScanNode.OperArt = OpMinus;break;
           case '+':    ScanNode.OperArt = OpPlus;break;
           case '/':    ScanNode.OperArt = OpDurch;break;
           case '*':    ScanNode.OperArt = OpMal;break;
           case '^':    ScanNode.OperArt = OpHoch;break;
           case ',':    ScanNode.OperArt = Komma;break;
           case 0  :    ScanNode.OperArt = SymEnde;break;   /* Zeile zu Ende */
        default :
          {
            ScanNode.OperArt = SymUnbekannt;
            error(errCharacter);
           } 
        }
        SkipZeichen();
        ClearNode(&ScanNode); /* sicherheitshalber durchloeschen */
    }
}


void StarteScanner(char *eingabe)
/* ---   Initialisiert den Scanner   --- */
{
  /* Reihenfolge Wichtig! */
    strcpy(ScanZeile,eingabe);        /* Eingabe  merken */
    ScanErrPos = 0;                   /* Startpositionen */
    ScanPosition = 0;
    ScanChar = ScanZeile[0];          /* Erstes Zeichen Holen */
    error(errOK);                     /* Fehler zuruecksetzen */
    LiesSymbol();                     /* erstes Symbol einlesen */
}

/*  ENDE SCANNER */

/* -------------------------------------------------------------------- */

void NewNode(PARSERPTR *node )
/* -----   Erzeugt einen neuen ParserNode   ----- */
{
    *node = malloc(sizeof (struct PARSERNODE));
    if (*node == NULL){                         /* kein Speicher: brutal raus! */
        fprintf(stderr,"Kein Speicher fuer neuen Parserknoten\n");
        exit(-105);
    }
    (*node)->OperArt = SymUnbekannt;            /* sicherheitshalber */
}


void LoescheBaum(PARSERPTR *p)
/* Loescht einen ParserBaum (gibt damit den Speicher wieder frei) */
{
  int i;
    if (*p != NULL)
    {
        switch ((*p)->OperArt)
        {
          case OpNeg:
                LoescheBaum(&(*p)->ops.Operand);break;
          case OpMinus :
          case OpPlus :
          case OpMal :
          case OpDurch:
          case OpHoch:
                LoescheBaum(&(*p)->ops.oper.nr1);
                LoescheBaum(&(*p)->ops.oper.nr2);break;
          case SymFunR1:
		for(i=0;i<(*p)->ops.funk.BezFktl->art.dim.fkt.va;i++)
		  LoescheBaum(&(*p)->ops.funk.Parameter[i]);
		break;
          case SymVarRR:
          case SymKoRR:break;
                /* Nix, gibt keine UnterBaeume */ 
          default :     /* da war wohl der Baum kaputt... */
                fprintf(stderr,"LoescheBaum: \"der Baum war wohl defekt\"\n");
                exit(-107);
        }
        free(*p);
        p = NULL;
    }
}

void SkipSymbol(void)
{
    if (SyntaxError!=true) LiesSymbol();
}

void BaueSymbol(PARSERPTR *nodePtr)
{
    if (SyntaxError){           /* Parser nicht richtig gestoppt */
        fprintf(stderr,"Parser nicht richtig gestoppt wegen Syntaxerror\n");
        exit(-108);
    }
    NewNode(nodePtr);
    **nodePtr = ScanNode;
    LiesSymbol();
}

void MussSymbol(SYMTYPE ErwartetesSymbol1,SYMTYPE ErwartetesSymbol2, ERRORTYPE err)
{
    if (ScanNode.OperArt == ErwartetesSymbol1 || ScanNode.OperArt == ErwartetesSymbol2)
       SkipSymbol();
    else   
       error(err);
}


/* ====================================================================
 * =================   2. der eigentliche Parser   ====================
 */


PARSERPTR makePrimary(void)
{
  int i;
    PARSERPTR node;
    if (SyntaxError) return NULL;
    switch (ScanNode.OperArt) 
    {
    case  Komma:
    case  KlammerAuf: /* ---- "(" <Term> ")" ---- */
            SkipSymbol();
            node = makeTerm();
            MussSymbol(KlammerZu,Komma, errKlammerZu);
            break; 
    case  OpMinus:    /* ---- "-" <Primary> ---- */
            BaueSymbol(&node);
            node->OperArt = OpNeg; /* das "-" hier monadisch */
            node->ops.Operand = makePrimary();
            break;
    case SymFunR1:   /* ---- <Bezeichner> <Primary> ---- */
            BaueSymbol(&node);
	    MussSymbol(KlammerAuf,KlammerAuf, errKlammerAuf);
/*	    if(ScanNode.OperArt==KlammerAuf) SkipSymbol(); */
	    for(i=0;i<node->ops.funk.BezFktl->art.dim.fkt.va;i++){
	      node->ops.funk.Parameter[i] = makeTerm();
	      MussSymbol(KlammerZu,Komma, errKlammerZu);
	    }
/*              node->ops.funk.Parameter[i] = makePrimary(); */
            break;
    case SymVarRR:
    case SymKoRR :     /* ---- Variable / Konstante ---- */
            BaueSymbol(&node);
            break;
    default:
        error(errAusdruck);
        return NULL;
    }
    return node;
}

PARSERPTR makeFaktor(void)
{
    PARSERPTR node, p;
    if  (SyntaxError) return NULL;
    node = makePrimary();
    if (ScanNode.OperArt == OpHoch)   /* ---- <Primary> "^" <Faktor> ---- */
    {
        BaueSymbol(&p);
        p->ops.oper.nr1 = node;
        p->ops.oper.nr2 = makeFaktor();
        node = p;
    }    
    return node;
}

PARSERPTR makeSummand(void)
{
    PARSERPTR node, p;
    if (SyntaxError) return NULL;
    node = makeFaktor();
    while (SyntaxError!=true)
    {
        switch (ScanNode.OperArt)
        {
           case OpMal   :
           case OpDurch :
                BaueSymbol(&p);
                p->ops.oper.nr1 = node;
                p->ops.oper.nr2 = makeFaktor();
                node = p;
                break;
           default : return node;
        }
    }
    return node;
}


PARSERPTR makeTerm(void)
{
    PARSERPTR node,p=NULL;
    if (SyntaxError) return NULL;
    node = makeSummand();
    while ( SyntaxError!=true)
    {
        switch (ScanNode.OperArt)
        {
           case OpPlus  :
           case OpMinus :
                BaueSymbol(&p);
                p->ops.oper.nr1 = node;
                p->ops.oper.nr2 = makeSummand();
                node = p;
                break;
           default : return node;
        }        
    }
    return node;
}
 

PARSERPTR parse( char *EingabeZeile)
{
    PARSERPTR node;
    
    StarteScanner(EingabeZeile);       /* Scanner Initialisieren */
    node = makeTerm();                 /* Einen Term Abholen  */
    if (ScanNode.OperArt != SymEnde){
        error(errOperator);
        LoescheBaum(&node);            /* Fehler ==> Baum wieder abbauen */
    }
    if(SyntaxError){
        fprintf(stderr,"Syntaxerror!\n");
        switch(ErrorArt){
          case errOK :         fprintf(stderr,"Kein Fehler\n");break;
          case errCharacter :  fprintf(stderr,"unerlaubtes Zeichen\n");break;
          case errBezeichner : fprintf(stderr,"undeklarierter Bezeichner\n");break;
          case errKlammerAuf : fprintf(stderr,"\"(\" erwartet\n");break; 
          case errKlammerZu :  fprintf(stderr,"\")\" erwartet\n");break; 
          case errKomma :      fprintf(stderr,"\",\" erwartet\n");break; 
          case errAusdruck :   fprintf(stderr,"Arithmetischer Ausdruck erwartet\n");break;
          case errOperator:    fprintf(stderr,"Operator erwartet\n");break; 
        }
        fprintf(stderr,"%s\n",EingabeZeile);
        for(i=1;i<ScanPosition;i++) fprintf(stderr,"-");
        fprintf(stderr,"|\n");
        exit(-120);
    }
    return node;
}
/*-----------------------------------------------------------------*/
/*-----------------------------------------------------------------*/
/* ====================================================================
 * ==============   3. der Rechner (berechnet den Baum)   =============
 */

double eval(PARSERPTR p)
{
    double schrott = 888.888;
    double   temp;
    ArithmeticError = false;     /* noch ist alles OK... */
    
    if (p == NULL){             /* Da ist wohl der Baum nicht vollstaendig */
        fprintf(stderr,"berchne: Der Baum ist nicht vollstaendig\n");
        exit(-109);
    }
    if (ArithmeticError) return schrott;  /* Bei Fehler: Ungueltig */

    switch (p->OperArt)
    {
      case OpNeg:
            return -eval(p->ops.Operand);
      case OpPlus:
            return eval(p->ops.oper.nr1) + eval(p->ops.oper.nr2);
      case OpMinus:
            return eval(p->ops.oper.nr1) - eval(p->ops.oper.nr2);
      case OpMal:
            return eval(p->ops.oper.nr1) * eval(p->ops.oper.nr2);
      case OpDurch:
            temp = eval(p->ops.oper.nr2);      /* Divisor zuerst Berechnen ... */
            if (temp == 0.0)                   /* Division durch Null abfangen */
            {
                ArithmeticError = true;
                return schrott;
            }
            return eval(p->ops.oper.nr1) / temp;
      case OpHoch:
            temp = eval(p->ops.oper.nr1);
            if (temp <= 0.0)               /* negative Basis abfangen */
            {
                ArithmeticError = true;
                return schrott;
            }    
            return pow(temp,eval(p->ops.oper.nr2));
      case SymPnode:
	    return eval(p->ops.funk.BezFktl->art.pnode);
      case SymFunR1:
	    switch(p->ops.funk.BezFktl->art.dim.fkt.va){
	    case 1:
            return p->ops.funk.BezFktl->art.dim.fkt.f( eval(p->ops.funk.Parameter[0]));
	    case 2:
            return p->ops.funk.BezFktl->art.dim.fkt2.f2( eval(p->ops.funk.Parameter[0]),
						  eval(p->ops.funk.Parameter[1]));
	    case 3:
            return p->ops.funk.BezFktl->art.dim.fkt3.f3( eval(p->ops.funk.Parameter[0]),
						  eval(p->ops.funk.Parameter[1]),
						  eval(p->ops.funk.Parameter[2]));
	    case 4:
            return p->ops.funk.BezFktl->art.dim.fkt4.f4( eval(p->ops.funk.Parameter[0]),
						  eval(p->ops.funk.Parameter[1]),
						  eval(p->ops.funk.Parameter[2]),
						  eval(p->ops.funk.Parameter[3]));
	    case 5:
            return p->ops.funk.BezFktl->art.dim.fkt5.f5( eval(p->ops.funk.Parameter[0]),
						  eval(p->ops.funk.Parameter[1]),
						  eval(p->ops.funk.Parameter[2]),
						  eval(p->ops.funk.Parameter[3]),
						  eval(p->ops.funk.Parameter[4]));
	    case 6:
            return p->ops.funk.BezFktl->art.dim.fkt6.f6( eval(p->ops.funk.Parameter[0]),
						  eval(p->ops.funk.Parameter[1]),
						  eval(p->ops.funk.Parameter[2]),
						  eval(p->ops.funk.Parameter[3]),
						  eval(p->ops.funk.Parameter[4]),
						  eval(p->ops.funk.Parameter[5]));
						}
      case SymVarRR:
            return p->ops.Variable->art.value;
      case SymKoRR:
            return p->ops.KoRR;
      default : exit(-11);  /* Baum defekt */
    }
    return schrott;
}

double berechne(PARSERPTR baum, double xWert)
{
    SetzeVariable(VarX,xWert);
    ArithmeticError = false;
    return eval(baum);
}


/*=================================================================*/
/********************* Optimiermodul *******************************/
/*******************************************************************/

typedef struct UMFORMUNG  *UMFORMUNGPTR;
struct UMFORMUNG 
       {
         UMFORMUNGPTR next;
         PARSERPTR   alt,neu;
        };


UMFORMUNGPTR ListeDerRegeln=NULL;
BOOLEAN  OptimierungWiederholen;


BOOLEAN konstant(PARSERPTR node)
{
    switch (node->OperArt)
    {
      case SymKoRR: return true;
      default : return false;
    }
}

BOOLEAN IsX(PARSERPTR node)
{
    return (node->OperArt == SymVarRR) && (node->ops.Variable == VarX);
}

BOOLEAN IsSymX(PARSERPTR node)
{
    return (node->OperArt == SymVarRR) && (node->ops.Variable == SymX);
}

void SetzeKonstant(PARSERPTR *node)
{
    double hw;
    hw = berechne(*node,999.999);
    LoescheBaum(node);
    NewNode(node);
    (*node)->OperArt = SymKoRR;
    (*node)->ops.KoRR = hw;
    OptimierungWiederholen = true;
}

PARSERPTR BaumCopy(PARSERPTR alt)
{
  int i;
    PARSERPTR neu;
    if (alt == NULL) exit (-20);
    NewNode(&neu);
    neu->OperArt = alt->OperArt;
    switch (alt->OperArt)
    {
      case OpNeg:
            neu->ops.Operand = BaumCopy(alt->ops.Operand);break;
      case OpMinus:
      case OpPlus :
      case OpDurch:
      case OpMal:
      case OpHoch:
            neu->ops.oper.nr1 = BaumCopy(alt->ops.oper.nr1);
            neu->ops.oper.nr2 = BaumCopy(alt->ops.oper.nr2);break;
      case SymKoRR:
      case SymVarRR:
            *neu = *alt;break;
      case SymFunR1:
            neu->ops.funk.BezFktl = alt->ops.funk.BezFktl;
	    for(i=0;i<alt->ops.funk.BezFktl->art.dim.fkt.va;i++)
	      neu->ops.funk.Parameter[i] = BaumCopy(alt->ops.funk.Parameter[i]);
	    break;
    }
    return neu;
}

/* Kopie mit einsetzen von 'ex' in 'alt' ------------------- */
PARSERPTR ErsetzeCopy(PARSERPTR alt,PARSERPTR ex)
{
  int i;
    PARSERPTR neu;
    if (alt == NULL) exit(-21);
    if (IsSymX(alt))  return BaumCopy(ex);
    NewNode(&neu);
    neu->OperArt = alt->OperArt;
    switch (alt->OperArt)
    {
      case OpNeg:
            neu->ops.Operand = ErsetzeCopy(alt->ops.Operand, ex);break;
      case OpMinus :
      case OpPlus :
      case OpDurch :
      case OpMal :
      case OpHoch:
            neu->ops.oper.nr1 = ErsetzeCopy(alt->ops.oper.nr1, ex);
            neu->ops.oper.nr2 = ErsetzeCopy(alt->ops.oper.nr2, ex);break;
      case SymFunR1:
            neu->ops.funk.BezFktl = alt->ops.funk.BezFktl;
	    for(i=0;i<alt->ops.funk.BezFktl->art.dim.fkt.va;i++)
	      neu->ops.funk.Parameter[i] = ErsetzeCopy(alt->ops.funk.Parameter[i], ex);
	    break;
      case SymKoRR :
      case SymVarRR:
            *neu = *alt;break;
/*      case SymVarRR:
            if(0==strcasecmp(alt->ops.Variable->Name,"X")){
                neu->OperArt=SymVarRR;
                neu->ops.Variable=VarX;
            } else {
            *neu = *alt;
            } */
    }
    return neu;
}

BOOLEAN OpGleich(PARSERPTR n1 ,PARSERPTR n2)
{
    if (n1->OperArt == n2->OperArt)
    {
        switch (n1->OperArt)
        {
          case SymFunR1:
                return n1->ops.funk.BezFktl == n2->ops.funk.BezFktl; 
          case SymKoRR:
                return n1->ops.KoRR == n2->ops.KoRR;
          case SymVarRR:
                return n1->ops.Variable == n2->ops.Variable;
          default : return true;
        }
    }    
    else
    {
        if (konstant(n1) && konstant(n2))
            return berechne(n1,123.456) == berechne(n2,321.654);
        else
            return false;
    }
}


BOOLEAN BaumGleich(PARSERPTR b1,PARSERPTR b2)
{
    if (b1 == NULL) exit(-22);
    if (b2 == NULL) exit(-23);
    if (OpGleich(b1,b2) == false) return false;
    switch (b1->OperArt)
    {
      case OpNeg:
            return BaumGleich(b1->ops.Operand, b2->ops.Operand);
      case OpPlus:
      case OpMal:    /* ---   kommutative Operatoren   --- */
            return (BaumGleich(b1->ops.oper.nr1, b2->ops.oper.nr1)
                && BaumGleich(b1->ops.oper.nr2, b2->ops.oper.nr2))
             ||    (BaumGleich(b1->ops.oper.nr1, b2->ops.oper.nr2)
                && BaumGleich(b1->ops.oper.nr2, b2->ops.oper.nr2));
      case OpMinus:
      case OpDurch:
      case OpHoch:
            return BaumGleich(b1->ops.oper.nr1, b2->ops.oper.nr1)
                && BaumGleich(b1->ops.oper.nr2, b2->ops.oper.nr2);
      case SymFunR1:
	    if(b1->ops.funk.BezFktl->art.dim.fkt.va != b2->ops.funk.BezFktl->art.dim.fkt.va)
	      return false;
	    for(i=0;i<b1->ops.funk.BezFktl->art.dim.fkt.va;i++)
	      if( false == BaumGleich(b1->ops.funk.Parameter[i], b2->ops.funk.Parameter[i])) 
		return false;
	    return true;
      default: return true;
    }
}

/* versucht, eine Regel aus der Liste anzuwenden ------------ */

    PARSERPTR FormX;

    BOOLEAN TesteRegel(PARSERPTR baum, PARSERPTR vgl)
    {
        if (IsSymX(vgl))
        {
            if (FormX == NULL)
            {
                FormX = baum; return true;
            }
            else return BaumGleich(baum, FormX);
        }
        else
        {
            if (OpGleich(baum, vgl))
               switch (baum->OperArt)
               {
                  case OpNeg:
                        return TesteRegel(baum->ops.Operand, vgl->ops.Operand);
                  case OpPlus:
                  case OpMal:      /* ---- kommutative Operatoren ---- */
                        return (TesteRegel(baum->ops.oper.nr1, vgl->ops.oper.nr1)
                             && TesteRegel(baum->ops.oper.nr2, vgl->ops.oper.nr2))
                         ||    (TesteRegel(baum->ops.oper.nr1, vgl->ops.oper.nr2)
                             && TesteRegel(baum->ops.oper.nr2, vgl->ops.oper.nr1));
                  case OpMinus :
                  case OpDurch :
                  case OpHoch :
                        return TesteRegel(baum->ops.oper.nr1, vgl->ops.oper.nr1)
                            && TesteRegel(baum->ops.oper.nr2, vgl->ops.oper.nr2);
                  case SymFunR1:
                        return TesteRegel(baum->ops.funk.Parameter[0], vgl->ops.funk.Parameter[0]);
                  default : return true;
                }
            else
                return false;
        }
    }
    
void NutzeRegel(PARSERPTR *formel, PARSERPTR alt, PARSERPTR neu)
{
    FormX = NULL;
    if (TesteRegel(*formel, alt))
    {
        if (FormX != NULL) FormX = BaumCopy(FormX);
        LoescheBaum(formel);        /* ...hier geloescht werden */
        *formel = ErsetzeCopy(neu, FormX);
        if (FormX != NULL) LoescheBaum(&FormX);
        OptimierungWiederholen = true;
    }
}

/* alle Regeln durchprobieren ------------ */
void ProbiereRegeln(PARSERPTR *baum)
{
    UMFORMUNGPTR up;
    PARSERPTR altbaum;
    do
    {
        altbaum = *baum;
        up = ListeDerRegeln;
        while (up != NULL)
        {
            NutzeRegel(baum, up->alt, up->neu);
            up = up->next;
        }
    } while (altbaum != *baum);    /* bis keine Regel mehr angewendet wurde */
}

PARSERPTR SchiftKonst(PARSERPTR baum,PARSERPTR konst,PARSERPTR rest)
{
    PARSERPTR p;
    
    OptimierungWiederholen=true;
    switch(rest->OperArt){
        case OpMal:
            p=rest->ops.oper.nr1;
            rest->ops.oper.nr1=baum;
            baum->ops.oper.nr1=konst;
            baum->ops.oper.nr2=p;
            return SchiftKonst(rest,konst,p);
    }
    
    return baum;
}

void optimize(PARSERPTR *baum)
{
  int i;
    do
    {
        if (ArithmeticError) return;
        /* -----   zuerst die UnterBaeume Optimieren   ----- */
        switch ((*baum)->OperArt)
        {
         case  OpNeg:
               optimize(&((*baum)->ops.Operand));break;
         case OpMinus:
         case OpPlus:
         case OpDurch:
         case OpMal:
         case OpHoch:
               optimize(&((*baum)->ops.oper.nr1));
               optimize(&((*baum)->ops.oper.nr2));break;
/*         case OpMal:
                if (konstant((*baum)->ops.oper.nr1) && konstant((*baum)->ops.oper.nr2))
                    SetzeKonstant(baum);
                else if(konstant((*baum)->ops.oper.nr1)){
                    merk=true;
                    *baum=SchiftKonst(*baum,(*baum)->ops.oper.nr1,(*baum)->ops.oper.nr2);
                    optimize(&((*baum)->ops.oper.nr2));break;
                }else if(konstant((*baum)->ops.oper.nr2)){
                    merk=true;
                    *baum=SchiftKonst(*baum,(*baum)->ops.oper.nr2,(*baum)->ops.oper.nr1);
                    optimize(&((*baum)->ops.oper.nr1));break;
                } else{
                   optimize(&((*baum)->ops.oper.nr1));
                   optimize(&((*baum)->ops.oper.nr2));break;
                }
                break;
  */              
         case SymFunR1:
	       for(i=0;i<(*baum)->ops.funk.BezFktl->art.dim.fkt.va;i++)
		 optimize(&((*baum)->ops.funk.Parameter[i]));
        }

        OptimierungWiederholen = false;
        ProbiereRegeln(baum);

        /* ------   konstante Ausdruecke berechnen   ----- */
        switch ((*baum)->OperArt)
        {
          case OpNeg:
                if (konstant((*baum)->ops.Operand)) SetzeKonstant((baum));break;
          case OpMinus:
          case OpPlus:
          case OpHoch:
          case OpDurch:
          case OpMal:
                if (konstant((*baum)->ops.oper.nr1) && konstant((*baum)->ops.oper.nr2))
                    SetzeKonstant(baum);
                break;
          case SymFunR1:
                if ((*baum)->ops.funk.BezFktl->art.dim.fkt.va==1 &&  konstant((*baum)->ops.funk.Parameter[0]))
                    SetzeKonstant(baum);
        }
    } while (OptimierungWiederholen);
}


void Optimiere(PARSERPTR *f)
{
    BEZPTR merk;
  
    if((*f)!=NULL){
        ArithmeticError = false;
        merk=VarX;
        VarX=HoleBezeichner("x");
        optimize(f);
        VarX=merk;
        if (ArithmeticError) LoescheBaum(f);
    }
}

void LerneRegel(char StrAlt[], char StrNeu[])
{
    UMFORMUNGPTR regel;
    regel = malloc(sizeof(struct UMFORMUNG));
    if (regel == NULL){               /* kein Speicher: brutal raus! */
        fprintf(stderr,"LerneRegel: Kein Speicher\n");
        exit(-111);
    }
    regel->next = ListeDerRegeln;
    regel->alt = parse(StrAlt); if (SyntaxError){
        fprintf(stderr,"LerneRegel: Syntaxerror\n");
        exit(-112);
    }
    regel->neu = parse(StrNeu); if (SyntaxError){
        fprintf(stderr,"LerneRegel: Syntaxerror\n");
        exit(-112);
    }
    ListeDerRegeln = regel;
}
 
/******************** Ende Optimiermodul ****************************/
/********************************************************************/

/******************* Begin DIFF Modul ******************************************/

void LerneAbleitung(BEZSTRING StrFkt, char *StrAbleit,int pnr)
{
    BEZPTR fkt;
    PARSERPTR d;
    DIFFERPTR r=NULL;
    
    fkt = HoleBezeichner(StrFkt);
    if(fkt == NULL){                        /* Funktion nicht bekannt */
        fprintf(stderr,"LerneAbleitung: Funktion %s nicht bekannt\n",StrFkt);
        exit(-113);
    }
    if(fkt->BezArt != SymFunR1) exit(-12); /* keine Funktion */
    d = parse(StrAbleit);
    if (SyntaxError) exit(-13);

    if(fkt->art.dim.fkt.va>1 && (d->OperArt != SymFunR1 || fkt->art.dim.fkt.va != d->ops.funk.BezFktl->art.dim.fkt.va)){
      fprintf(stderr,"Die Ableitung von mehrparametrischen Funktionen muss wieder eine solche sein\n");
      exit(-272);
    }

    if(pnr<1 || pnr>fkt->art.dim.fkt.va){           /* Ableitungsnummer Ueberpruefen */
        fprintf(stderr,"Ungueltige Ableitungsnummer %d\n",pnr);
        exit(-522);
  }
/* Suchen ob schon ein Eintrag vorhanden */
    r = ListeDerAbleitungen;
    while ((r != NULL) && (r->Funktion != fkt)) r = r->next;

    if(r==NULL){  /* => noch kein Eintrag */
      r=malloc(sizeof(struct DIFFER));
      r->next = ListeDerAbleitungen;         /* in die Liste eintragen */
      r->Funktion = fkt;
      ListeDerAbleitungen = r;
    }
    r->Ableitung[pnr-1] = d;  /* Ableitung eintragen */
}

PARSERPTR HoleAbleitung(BEZPTR fkt,int pnr)
{
/* --- sucht die Ableitung einer Funktion aus der Liste --- */
    DIFFERPTR r;

    r = ListeDerAbleitungen;
    while ((r != NULL) && (r->Funktion != fkt)) r = r->next;
    if (r == NULL) return NULL; else return r->Ableitung[pnr];
}

PARSERPTR operator(SYMTYPE op, PARSERPTR a, PARSERPTR b)
{
    PARSERPTR p;

    NewNode(&p);
    p->OperArt = op;
    p->ops.oper.nr1 = a;
    p->ops.oper.nr2 = b;
    return p;
}

PARSERPTR plus(PARSERPTR a,PARSERPTR b)
{
    return operator(OpPlus,a,b);
}

PARSERPTR minus(PARSERPTR a,PARSERPTR b)
{
    return operator(OpMinus, a,b);
}

PARSERPTR mal(PARSERPTR a,PARSERPTR b)
{
    return operator(OpMal,a,b);
}

PARSERPTR durch(PARSERPTR a,PARSERPTR b)
{
    return operator(OpDurch,a,b);
}

PARSERPTR hoch(PARSERPTR a,PARSERPTR b)
{
    return operator(OpHoch,a,b);
}

PARSERPTR BaueLn(PARSERPTR a)
{   PARSERPTR p;

    if (LnFunktion == NULL) LnFunktion = HoleBezeichner("LN");
    if (LnFunktion == NULL){ /* Eine Funktion "LN" mu da sein !!! */
       DifferError = true;
       return NULL;
    } else {
        NewNode(&p);
        p->OperArt = SymFunR1;
        p->ops.funk.BezFktl = LnFunktion;
        p->ops.funk.BezFktl->art.dim.fkt.va=1;
        p->ops.funk.Parameter[0] = a;
        return p;
    }
}
PARSERPTR BaueNull(void)
{   PARSERPTR p;
        NewNode(&p);
        p->OperArt = SymKoRR;
        p->ops.KoRR=0;
        return p;
}

PARSERPTR konst(double x)
{   PARSERPTR p;

    NewNode(&p);
    p->OperArt = SymKoRR;
    p->ops.KoRR = x;
    return p;
}

PARSERPTR diff(PARSERPTR p)
{   PARSERPTR d,ptr;
    int i,k;
    
    if (DifferError) return NULL;
    switch(p->OperArt){
    case OpNeg:
      return minus(konst(0.0), diff(p->ops.Operand));
    case OpMinus:
      return minus(diff(p->ops.oper.nr1), diff(p->ops.oper.nr2));
    case OpPlus:
      return plus(diff(p->ops.oper.nr1), diff(p->ops.oper.nr2));
    case OpMal:
      /* --- (f*g)' = f'*g + f*g' ---  */
      return plus(
		  mal(diff(p->ops.oper.nr1), BaumCopy(p->ops.oper.nr2)),
		  mal(BaumCopy(p->ops.oper.nr1), diff(p->ops.oper.nr2)));
    case OpDurch:
      /* --- (f/g)' = (f'*g - f*g')/(g*g)--- */
      return durch(
		   minus (
			  mal(diff(p->ops.oper.nr1),BaumCopy(p->ops.oper.nr2)),
			  mal(BaumCopy(p->ops.oper.nr1),diff(p->ops.oper.nr2))),
		   mal (BaumCopy(p->ops.oper.nr2), BaumCopy(p->ops.oper.nr2))
		   );
    case OpHoch:
      d = BaumCopy(p->ops.oper.nr2);
      Optimiere(&d);
      if(konstant(d))                                     
	/* --- (f^k)' = k*f' * f^(k-1) --- */
	return mal(
		   mal(d, diff(p->ops.oper.nr1)),
		   hoch(
			BaumCopy(p->ops.oper.nr1),
			minus(BaumCopy(d), konst(1.0))
			) );
      else
	/* --- (f^g)' = f^g * (g'*ln f + g*f'/f)  --- */
	return mal(
		   hoch(BaumCopy(p->ops.oper.nr1),BaumCopy(d)),
		   plus(
			mal(diff(d), BaueLn(BaumCopy(p->ops.oper.nr1))),
			durch(mal(d, diff(p->ops.oper.nr1)),
			      BaumCopy(p->ops.oper.nr1))
			) ); 
    case SymKoRR:
    case SymVarRR:
      if(IsX(p)) return konst(1.0);
      else return konst(0.0);
    case SymFunR1:
      if(p->ops.funk.BezFktl->art.dim.fkt.va==1){  /* einparametrige Funktionen */
	d = HoleAbleitung(p->ops.funk.BezFktl,0);
	if (d == NULL){
	  DifferError = true;
	  return NULL;
	}
	else  /* df/da*da/dx+df/db*db/dx... */
	  return mal(diff(p->ops.funk.Parameter[0]),
		     ErsetzeCopy(d,p->ops.funk.Parameter[0]) );
      }else{                                  /* mehrparametrige Funktionen */
	ptr=BaueNull();
	for(i=0;i<p->ops.funk.BezFktl->art.dim.fkt.va;i++){
	  d = HoleAbleitung(p->ops.funk.BezFktl,i);
	  if (d == NULL){
	    DifferError = true;
	    return NULL;
	  }
	   /* df/da*da/dx+df/db*db/dx... SIGI */
	  for(k=0;k<p->ops.funk.BezFktl->art.dim.fkt.va;k++)  /* Parameter einsetzen */
	    d->ops.funk.Parameter[k]=p->ops.funk.Parameter[k];
	    
	  ptr=plus( mal(diff(p->ops.funk.Parameter[i]) , BaumCopy(d) ) ,
		     ptr);
	}
	Optimiere(&ptr);
	return ptr;
      }
    default:
      DifferError = true;
      return NULL;
    }
  }

PARSERPTR Ableitung(PARSERPTR baum,char *variable)
{
    PARSERPTR d;

    VarX=HoleBezeichner(variable);
    if (baum == NULL){
        fprintf(stderr,"Ableitung: Funktionspointer ist NULL\n");
        exit(-114);
    }
    DifferError = false;
    d = diff(baum);
    if (DifferError)
        LoescheBaum(&d);
    else
        Optimiere(&d);

    VarX=HoleBezeichner("x"); /* X-bezeichner zuruecksetzen */

    return d;
}
/******************* Ende DIFF Modul ******************************************/



/******************** Math-Lehrer-Modul *****************************/
/********************************************************************/

void LehreKonstanten(void)
{
  LerneVariable("PI", 3.14159265358979328);
  LerneVariable("E", exp(1.0));
  LerneVariable("X", 1);
}

double sqr(double x)
{
    return x*x;
}

double sign(double x)
{
    if(x>0) return 1;
    else if(x<0) return -1;
    else return 0;
}

void LehreFunktionen(void)
{
  LerneFunktion("SIN", sin,1);
  LerneFunktion("COS", cos,1);
  LerneFunktion("TAN", tan,1);
  LerneFunktion("ARCTAN", atan,1);
  
  LerneFunktion("SINH", sinh,1);
  LerneFunktion("COSH", cosh,1);
  LerneFunktion("TANH", tanh,1);

  LerneFunktion("LN", log,1);
  LerneFunktion("EXP", exp,1);
  LerneFunktion("SQRT", sqrt,1);
  LerneFunktion("SQR", sqr,1);
  LerneFunktion("ABS", fabs,1);
  LerneFunktion("SIGN", sign,1);
}



void LehreRegeln(void)
{
  SymX=VarX=HoleBezeichner("X");
  
  LerneRegel("TAN (ARCTAN (X))", "X");
  LerneRegel("SIN (X) / COS (X)", "TAN (X)");
  LerneRegel("SQR (SIN (X)) + SQR (COS (X))", "1");
  LerneRegel("SIN (0)", "0");
  LerneRegel("COS (0)", "1");
  LerneRegel("SIN (PI)", "0");
  LerneRegel("COS (PI)", "-1");

  LerneRegel("SQR (COSH (X)) - SQR (SINH (X))", "1");

  LerneRegel("EXP (LN (X))", "X");    
  LerneRegel("LN (EXP (X))", "X");
  LerneRegel("E^X", "EXP (X)");

  LerneRegel("SQRT (SQR (X))", "ABS (X)");
  LerneRegel("SQR (SQRT (X))", "X"); 
  LerneRegel("X*X", "SQR (X)");
  LerneRegel("X^2", "SQR (X)");
  LerneRegel("X^1", "X");

  LerneRegel("--X", "X");
  LerneRegel("0+X", "X");
  LerneRegel("X-0", "X");
  LerneRegel("0-X", "-X");
  LerneRegel("X-X", "0");

  LerneRegel("0*X", "0");
  LerneRegel("1*X", "X");
  LerneRegel("0/X", "0");
  LerneRegel("X/X", "1");
  LerneRegel("X/0", "1/0");
}


void LehreAbleitungen(void)
{
  LerneAbleitung("SIN", "COS(X)",1);
  LerneAbleitung("COS", "-SIN(X)",1);
  LerneAbleitung("TAN","1/SQR(COS(X))",1);
  LerneAbleitung("ARCTAN", "1/(1+SQR(X))",1);

  LerneAbleitung("SINH", "COSH(X)",1);
  LerneAbleitung("COSH", "SINH(X)",1);
  LerneAbleitung("TANH", "1/SQR(COSH(X))",1);

  LerneAbleitung("EXP", "EXP(X)",1);
  LerneAbleitung("LN", "1/X",1);

  LerneAbleitung("SQRT", "1/(2*SQRT(X))",1);
  LerneAbleitung("SQR", "2*X",1);
  LerneAbleitung("ABS", "SIGN(X)",1);
  LerneAbleitung("SIGN", "0",1);
}


void LehreMathe(void)
{
  LehreFunktionen();
  LehreKonstanten();
  LehreAbleitungen();
  LehreRegeln();
}

/****************** Ende Math-Lehrer *************************************/
/*************************************************************************/


/*

void main(int argc,char* argv[])
{
  char str[4096];
  
  PARSERPTR fptr;
  LehreMathe();

   

  if(argc>1){
    fptr=parse(argv[1]);
    BaumZuString(fptr,str);
    printf("nor: %s=%lg\n",str,eval(fptr));
    printf("nor: %s=%lg\n",str,eval(fptr));
    Optimiere(&fptr);
    SetzeVariable(HoleBezeichner("x"),1);
    BaumZuString(fptr,str);
    printf("opt: %s=%lg\n",str,eval(fptr));

    fptr=Ableitung(fptr,"x");
    Optimiere(&fptr);
    BaumZuString(fptr,str);
    printf("abl: %s\n",str);

  }


}
*/


