/**************************************************************/
/*                                                            */
/*         Modul zum Parsen von Funktionsstrings              */
/*                                                            */
/* 11.09.91 Siegfried Goetz:                                  */
/*           Implementation durch Umsetzung von Modula-Modul  */
/*           aus dem Buch "MODULA-2",S.Duerholt,J.Schnur,     */
/*           Markt&Technik in Turbo-C                         */
/*                                                            */
/* 26.10.93 Siegfied Goetz:                                   */
/*           Erweiterung der Differentiationsmoeglichkeiten   */
/*           auf alle Variablen.                              */
/*           Verbesserung der Fehlermeldungsausgabe           */
/*                                                            *
   25.3.95 Siegfried Goetz:
             Es koennen jetzt auch mehrdimensionale Funktionen
             gelernt werden incl. die Dazugehoerigen Ableitungen
             zu jedem Parameter.
  **************************************************************/

typedef enum
        {
          errOK,         /* Kein Fehler */
          errCharacter,  /* unerlaubtes Zeichen im Text */
          errBezeichner, /* undeklarierter Bezeichner */
          errKlammerAuf, /* "(" erwartet */
          errKlammerZu,  /* ")" erwartet */
          errKomma,      /* "," erwartet */
          errAusdruck,   /* Arithmetischer Ausdruck erwartet */
          errOperator    /* Operator erwartet */
        } ERRORTYPE;

extern int  SyntaxError;     /* ist = 0 wenn beim Parsen kein Fehler aufgetreten ist */
extern int  ArithmeticError; /* ist = 0 wenn beim Berechnen eines Baumes kein Fehler aufgetreten ist */
extern ERRORTYPE  ErrorArt;  /* gibt den Fehlertyp an bei einem Syntaxfehler */


/**************************************************************/
/*                                                            */
/*        LerneFunktion()                                     */
/*                                                            */
/* Mit dieser Funktion kann dem Parser eine Funktion bekannt  */
/* bekannt gemacht werden. Es mss  der Funktionsstring und die*/
/* Addresse der Funktion Uebergeben werden. Eine Funktion muss*/
/* immer auf folgende weise definiert sein:                   */
/* double funkt(double,...)                                   */        
/* anz ist die Zahl der Parameter                             */
/**************************************************************/

void LerneFunktion( char *name,const void *funkt,int panz);


/**************************************************************/
/*                                                            */
/*                   LerneVariable()                          */
/*                                                            */
/* Mit dieser Funktion knnen dem Parser Variablen/Konstanten */
/* bekanntgemacht werden.Der Namensstring sollte nicht mehr   */
/* wie 15 Zeichen lang sein.                                  */
/*                                                            */
/**************************************************************/

void LerneVariable(char *name, double wert);


/**************************************************************/
/*                                                            */
/*                  HoleBezeichner()                          */
/*                                                            */
/*   Mit diese Funktion liefert einen Pointer auf den Bezeich-*/
/*   nerstruct einer Variablen. Dieser wird bentigt um den  */
/*   Wert einer Variablen zu aendern.                         */          
/*                                                            */
/**************************************************************/

void *HoleBezeichner(char *name);


/**************************************************************/
/*                                                            */
/*              SetzeVariable()                               */
/*                                                            */
/*   Mit dieser Funktion kann man den Wert einer Variablen    */
/*   geaendert werden, durch Uebergabe des Bezeichnerpointers */
/*   und des neuen Wertes.                                    */ 
/*                                                            */
/**************************************************************/

void SetzeVariable(void *bez, double wert);


/**************************************************************/
/*                                                            */
/*         LehreMathe()                                       */
/*                                                            */
/* Diese Funktion lehrt den Parser folgende Funktionen:       */
/*   SIN,COS,TAN,ARCTAN,SINH,COSH,TANH,LN,EXP,SQRT,ABS        */
/*   SIGN                                                     */
/* und folgende Konstanten:                                   */
/* PI,E                                                       */
/* Es werden auch alle Ableitungsregeln gelernt und ein       */
/* paar Optimierungsregeln.                                   */
/*                                                            */
/**************************************************************/

void LehreMathe(void);


/**************************************************************/
/*                                                            */
/*         parse()                                            */
/*                                                            */
/* Diese Funktion ist der eigentliche Parser und bildet einen */
/* Parser-Baum aus dem Funktionsstring. Als Ergebnis wird ein */
/* Pointer zu diesem Baum zucueckgeliefert, welcher spaeter bei*/
/* der Berechnung benoetigt wird.                             */ 
/*                                                            */
/**************************************************************/

void  *parse( char *Funktionsstring);

/**************************************************************/
/*                                                            */
/*         Ableitung()                                        */
/*                                                            */
/* Diese Funbktion leitet den Parserbaum formel nach der      */
/* Variablen var ab. Es wird ein neuer Baum gebildet, und     */
/* dessen Pointer zurueckgeliefert. Der Baum formel wird      */
/* nicht veraendert.                                          */ 
/*                                                            */
/**************************************************************/

void  *Ableitung(void *formel,char *var);

/**************************************************************/
/*                                                            */
/*         LerneAbleitung()                                   */
/*                                                            */
/* Diese Funbktion brigt dem Differenzierer eine Ableitung    */
/* bei. z.B. StrFkt: "cos" und StrAbleit = "-sin"             */
/* Die Funktionen muessen dem Parser bereits mit LerneFunktion*/
/* beigeracht worden sein.                                    */
/* Parnr ist die Nummer des Parameters nach dem die Ableitung */
/* berechnet ist. (normalerweise 1). Bei n-dimenionalen Fkt.  */
/* muessen den Parser auch n Ableitungen beigebracht werden um*/
/* mit dem Ableitungsmodul zu arbeiten.                       */
/**************************************************************/

void LerneAbleitung(char *StrFkt, char *StrAbleit,int Parnr);

/**************************************************************/
/*                                                            */
/*         Optimiere()                                        */
/*                                                            */
/* Diese Funbktion versucht den Funktionsbaum formel zu       */
/* optimeren. Diese Funktion wird automatisch nach auf jeden  */
/* Ableitungsbaum angewendet.                                 */
/*                                                            */
/**************************************************************/
void Optimiere(void **formel);

/**************************************************************/
/*                                                            */
/*         LoescheBaum()                                      */
/*                                                            */
/* Diese Funktion loescht einen Parserbaum wieder und gibt den*/
/* dafuer belegten Speicher wieder frei. Es muss dazu die Add-*/
/* resse des Baumpointers uebergeben werden.                  */
/*                                                            */
/**************************************************************/

void LoescheBaum(void **p);


/**************************************************************/
/*                                                            */
/*         berechne()                                         */
/*                                                            */
/* Berechnet einen Parserbaum und liefert das Ergebnis. ES    */
/* muss ein Pointer auf dem Parserbaum uebergeben werden.     */   
/*                                                            */
/**************************************************************/

double berechne(void *p,double xWert);


/**************************************************************/
/*                                                            */
/*    BaumZuString()                                          */
/*                                                            */
/* Diese Funktion wandelt einen Parserbaum in einen String    */
/* zurueck. Es muss ein Pointer auf einen genuegend grossen   */
/* Speicherbereich uebergeben werden und der Pointer auf den  */
/* Parserbaum.                                                */
/*                                                            */
/**************************************************************/

void BaumZuString(void *baum, char *formelstr);


/**************************************************************/
/*                                                            */
/*    PrinTree()                                              */
/*                                                            */
/* Diese Funktion gibt den Parserbaum auf den Bildschirm aus. */
/*                                                            */
/**************************************************************/

void PrinTree(void *baum);

