
/****************************************************************************
 *    				    					    							*
 *    DATEN-Modul von Siegfried Goetz					    				*
 *    				    					    							*
 *    letzte Aenderung 02.02.93						    					*
 *    kleine Anpassungen 07.01.98 Andreas Wrl								*
 *    				    					    							*
 ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include "unistd.h"
#include "rwadat.h"

/****************************************************************************
*    				    					    							*
*    chabyte:  vertauscht die Bytereihenfolge eines Speicherbereichs.	    *
*    				    					    							*
*    EINGABEN: ptr ist ein Pointer auf den Speicherbereich.		    		*
*    	       anz ist die Anzahl der Bytes die vertauscht werden soll.	    *
*    ERGEBNIS: keines.							    						*
*    				    					    							*   											  
****************************************************************************/
void chabyte(void *ptr, long anz)
{
#if BYTE_ORDER == LITTLE_ENDIAN
	long i;
	unsigned char m, *p;
	p = (unsigned char *) ptr;

	for (i = 0; i < anz / 2; i++) {
		m = p[i];
		p[i] = p[anz - 1 - i];
		p[anz - 1 - i] = m;
	}
#endif
}

static int rsbyte(void *ptr, int count, char typ, double norm, double off, FILE * stream)
{
	char data[2048];
	int anz, j, i;
	double *dptr;
	float *fptr;
	int *lptr;
	short int *iptr, fehlnr = 0;
	unsigned char *bptr;

	dptr = (double*) ptr;
	fptr = (float*) ptr;
	lptr = (int*) ptr;
	iptr = (short int*) ptr;
	bptr = (unsigned char*) ptr;

	for (i = 0; i < count;) {
		anz = fread(data, 1, MIN((count - i), 2048), stream);	/* Daten einlesen */
		for (j = 0; j < anz; j++) {
			switch (typ) {
			case 'D':
				*dptr++ = (double) (norm * data[j]) + off;
				break;
			case 'F':
				*fptr++ = (float) (norm * (double) data[j] + off);
				break;
			case 'L':
				*lptr++ = (int) (data[j]);
				break;
			case 'W':
				*iptr++ = (short int) (data[j]);
				break;
			case 'B':
				*bptr++ = (unsigned char) (data[j]);
				break;
			default:
				fehlnr = -5, i = count;
			}
			i++;
			if (i >= count)
				break;
		}
	}
	return fehlnr;
}

static int rbyte(void *ptr, int count, char typ, double norm, double off, FILE * stream)
{
	unsigned char data[2048];
	int anz, j, i;
	double *dptr;
	float *fptr;
	int *lptr;
	short int *iptr, fehlnr = 0;
	unsigned char *bptr;

	dptr = (double *)ptr;
	fptr = (float *)ptr;
	lptr = (int *)ptr;
	iptr = (short int*)ptr;
	bptr = (unsigned char*) ptr;

	for (i = 0; i < count;) {
		anz = fread(data, 1, MIN((count - i), 2048), stream);	/* Daten einlesen */
		for (j = 0; j < anz; j++) {
			switch (typ) {
			case 'D':
				*dptr++ = (double) (norm * data[j]) + off;
				break;
			case 'F':
				*fptr++ = (float) (norm * (double) data[j] + off);
				break;
			case 'L':
				*lptr++ = (int) (data[j]);
				break;
			case 'W':
				*iptr++ = (short int) (data[j]);
				break;
			case 'B':
				*bptr++ = (unsigned char) (data[j]);
				break;
			default:
				fehlnr = -5, i = count;
			}
			i++;
			if (i >= count)
				break;
		}
	}
	return fehlnr;
}

static int rint(void *ptr, int count, char typ, double norm, double off, FILE * stream)
{
	short int data[256];
	int anz, j, i;
	double *dptr;
	float *fptr;
	int *lptr;
	short int *iptr;
	int fehlnr = 0;

	dptr = (double*)ptr;
	fptr = (float*)ptr;
	lptr = (int*)ptr;
	iptr = (short int*)ptr;

	for (i = 0; i < count;) {
		anz = fread(data, 2, MIN((count - i), 256), stream);	/* Daten einlesen */
		for (j = 0; j < anz; j++)	/* Bytes vertauschen */
			chabyte(&(data[j]), 2);
		for (j = 0; j < anz; j++) {
			switch (typ) {
			case 'D':
				*dptr++ = (double) (norm * data[j]) + off;
				break;
			case 'F':
				*fptr++ = (float) (norm * (double) data[j] + off);
				break;
			case 'L':
				*lptr++ = (int) (data[j]);
				break;
			case 'W':
				*iptr++ = (short int) (data[j]);
				break;
			default:
				fehlnr = -5, i = count;
			}
			i++;
			if (i >= count)
				break;
		}
	}
	return fehlnr;
}

static int rlong(void *ptr, int count, char typ, double norm, double off, FILE * stream)
{
	int data[256];
	int anz, j, i;
	double *dptr;
	float *fptr;
	int *lptr;
	int fehlnr = 0;

	dptr = (double*)ptr;
	fptr = (float*)ptr;
	lptr = (int*)ptr;

	for (i = 0; i < count;) {
		anz = fread(data, 4, MIN((count - i), 256), stream);	/* Daten einlesen */
		for (j = 0; j < anz; j++)	/* Bytes vertauschen */
			chabyte(&(data[j]), 4);
		for (j = 0; j < anz; j++) {
			switch (typ) {
			case 'D':
				*dptr++ = (double) (norm * data[j]) + off;
				break;
			case 'F':
				*fptr++ = (float) (norm * (double) data[j] + off);
				break;
			case 'L':
				*lptr++ = (int) (data[j]);
				break;
			default:
				fehlnr = -5, i = count;
			}
			i++;
			if (i >= count)
				break;
		}
	}
	return fehlnr;
}

static int rfloat(void *ptr, int count, char typ, double norm, double off, FILE * stream)
{
	float data[256];
	int anz, j, i;
	double *dptr;
	float *fptr;
	int fehlnr = 0;

	dptr = (double*)ptr;
	fptr = (float*)ptr;

	for (i = 0; i < count;) {
		anz = fread(data, sizeof(float), MIN((count - i), 256), stream);	/* Daten einlesen */
		for (j = 0; j < anz; j++)	/* Bytes vertauschen */
			chabyte(&(data[j]), sizeof(float));	/*CHANGED!*/
		for (j = 0; j < anz; j++) {
			switch (typ) {
			case 'D':
				*dptr++ = (double) (norm * data[j]) + off;
				break;
			case 'F':
				*fptr++ = (float) (norm * (double) data[j] + off);
				break;
			default:
				fehlnr = -5, i = count;
			}
			i++;
			if (i >= count)
				break;
		}
	}
	return fehlnr;
} /* rfloat() */

static int rdouble(void *ptr, int count, char typ, double norm, double off, FILE * stream)
{
	double data[254];
	int anz, i;
	int j;
	double *dptr;
	float *fptr;
	int fehlnr = 0;

	dptr = (double*)ptr;
	fptr = (float*)ptr;

	for (i = 0; i < count;) {
		anz = fread(data, sizeof(double), MIN((count - i), 254), stream);	/* Daten einlesen */
		for (j = 0; j < anz; j++) /* Bytes vertauschen */
			chabyte(&(data[j]), sizeof(double));
		for (j = 0; j < anz; j++) {
			switch (typ) {
			case 'D':
				*dptr++ = (double) (norm * data[j]) + off;
				break;
			case 'F':
				*fptr++ = (float) (norm * data[j] + off);
				break;
			default:
				fehlnr = -5, i = count;
			}
			i++;
			if (i >= count)
				break;
		}
	}
	return fehlnr;
}

static int rddat(FILE * stream, int count, void *dat, char typ, double norm, double off, char dtyp)
{
	int fehlnr = 0;
	
	switch (dtyp) {				/* einlesen */
	case 'b':
		fehlnr = rsbyte(dat, count, typ, norm, off, stream);
		break;
	case 'B':
		fehlnr = rbyte(dat, count, typ, norm, off, stream);
		break;
	case 'W':
		fehlnr = rint(dat, count, typ, norm, off, stream);
		break;
	case 'L':
		fehlnr = rlong(dat, count, typ, norm, off, stream);
		break;
	case 'F':
		fehlnr = rfloat(dat, count, typ, norm, off, stream);
		break;
	case 'D':
		fehlnr = rdouble(dat, count, typ, norm, off, stream);
		break;
	default:
		return -3;
	}
	return fehlnr;
}

static int rdkopf(FILE * stream, double *tleft, double *tright, int *count,
				  double *normx, double *normy, char *xeinh, char *yeinh,
				  char *sinfo, char *dxtyp, char *dytyp, int *dszahl, char *stuc,
				  double *offx, double *offy)
{
	VORSPANN vs;
	DATENSATZ ds[2];
	int i;

	*offx = 0.0;
	*offy = 0.0;
	fseek(stream, 0, 0);
	fread(&vs, 1, sizeof(VORSPANN), stream);	/* Vorspann einlesen */
	chabyte(&(vs.datzahl), 2);

	if (vs.datzahl > 2)
		return -3;
	if (sinfo != NULL)
		*sinfo = vs.sinfo;
	*dxtyp = *dytyp = vs.dtyp;
	*dszahl = vs.datzahl;
	*stuc = vs.stuc;

	for (i = 0; (i < vs.datzahl) && (i < 2); i++) {		/* Datensaetze einlesen */
		fread(&(ds[i]), 1, sizeof(DATENSATZ), stream);
		chabyte(&(ds[i].norm), 4);
		chabyte(&(ds[i].tleft), 4);
		chabyte(&(ds[i].tright), 4);
		chabyte(&(ds[i].count), 4);
	}

	*count = ds[0].count;
	*tleft = (double) (ds[0].tleft);
	*tright = (double) (ds[0].tright);

	if (xeinh != NULL) {		/* Einheiten ermitteln */
		if (vs.datzahl > 1)
			strcpy(xeinh, ds[0].einh);
		else
			strcpy(xeinh, ds[vs.datzahl - 1].teinh);
	}
	if (yeinh != NULL)
		strcpy(yeinh, ds[vs.datzahl - 1].einh);

	/* Normierung bestimmen */
	*normy *= ds[vs.datzahl - 1].norm;
	if (vs.datzahl > 1)
		*normx *= ds[0].norm;
	return 0;
}

static int rdkopfn(FILE * stream, double *tleft, double *tright, int *count,
				   double *normx, double *normy, char *xeinh, char *yeinh,
				   char *sinfo, char *dxtyp, char *dytyp, int *dszahl,
				   char *stuc, double *offx, double *offy)
{
	VORSP vs;
	DASATZ ds[2];
	int i;

	*offx = 0.0;
	*offy = 0.0;
	
	
    if (sizeof(VORSP) != 40) {
		printf("WARNING: sizeof(VORSP)=%i  != 40\n", sizeof(VORSP));
	}
	
	fseek(stream, 0l, 0);
	fread(&vs, 1, sizeof(VORSP), stream);	/* Vorspann einlesen */

	chabyte(&(vs.datzahl), 2);
	chabyte(&(vs.count), 4);
	chabyte(&(vs.tleft), 8);
	chabyte(&(vs.tright), 8);
	
	if (vs.datzahl > 2)
		return -3;
	if (sinfo != NULL)
		*sinfo = vs.sinfo;
	*count = vs.count;
	*dszahl = vs.datzahl;
	*stuc = vs.stuc;
	*tleft = vs.tleft;
	*tright = vs.tright;

	if (sizeof(DASATZ) != 24) {
		printf("WARNING: sizeof(DASATZ)=%i  != 24\n", sizeof(DASATZ));
	}
	for (i = 0; (i < vs.datzahl) && (i < 2); i++) {		/* Datensaetze einlesen */
		fread(&(ds[i]), 1, sizeof(DASATZ), stream);
		chabyte(&(ds[i].offset), 8);
		chabyte(&(ds[i].norm), 8);
	}

	*dxtyp = ds[0].dtyp;
	*dytyp = ds[i - 1].dtyp;

	if (xeinh != NULL) {		/* Einheiten ermitteln */
		if (vs.datzahl > 1) {
			strcpy(xeinh, ds[0].einh);
			*offx = ds[0].offset;
		} else {
			strcpy(xeinh, vs.teinh);
			*offx = 0.0;
		}
	}
	if (yeinh != NULL)
		strcpy(yeinh, ds[*dszahl - 1].einh);
	*offy = ds[*dszahl - 1].offset;

	/* Normierung bestimmen */
	*normy = ds[*dszahl - 1].norm;
	if (vs.datzahl > 1)
		*normx = ds[0].norm;
	return 0;
}

#define NEWLN(a)  m=' '; while(m!='\n' && m!=EOF) m=fgetc(a);

int rdascii(void **x, void **y, char typ, char *fname, int start, int *count,
			double dx, double dy, char *sinfo,
			char *siptr, int *silen)
{
	FILE *stream;
	int j, i, ret;
	int tsi, mflag = 0;
	double *dxptr, *dyptr;
	float *fxptr, *fyptr;
	char xs[256], ys[256], m;

	sinfo[0] = 0;				/* keine Sonderinfo */
	siptr = 0;
	*silen = 0;

	switch (typ) {
	case 'F':
		tsi = sizeof(float);
		break;
	case 'D':
		tsi = sizeof(double);
		break;
	default:
		printf("Der Type %c ist bei ascii-Daten noch nicht implementiert\n", typ);
		exit(-943);
	}

	if (fname[0] == '<') 
		stream = popen(fname, "r");
	else
		stream = fopen(fname, "r");
	
	if (stream == NULL) {
		printf("Konnte Inputfile %s nicht oeffnen\n", fname);
		exit(-592);
	}
	if (*count == 0l)			/* Anzahl der Daten holen */
		*count = 65536;

	if (*x == NULL) {			/* Speicher anfordern */
		mflag = 1;
		*y = malloc(tsi * (*count));
		*x = malloc(tsi * (*count));
		if (*x == NULL) {		/* Zu wenig Speicher */
			if (fname[0] == '<')
				pclose(stream);
			else
				fclose(stream);
			return -443;
		}
	}
	dxptr = (double*)*x;
	fxptr = (float*)*x;
	dyptr = (double*)*y;
	fyptr = (float*)*y;

	j = 0, i = 0;
	do {						/* Daten einlesen */
		ret = fscanf(stream, "%s %s", xs, ys);
		if (xs[0] != '#' && ret == 2) {		/* Leerzeilen und Kommentare auslassen */
/*  	    w=strtod(xs,&fp); *//* Auf Gueltigkeit testen */
/*  	    if(w==0.0 && fp==xs){ 
 *				break;
 *			} */
			if (j >= start) {
				switch (typ) {
				case 'D':
					*dxptr++ = dx * atof(xs);
					*dyptr++ = dy * atof(ys);
					break;
				case 'F':
					*fxptr++ = (float) (dx * atof(xs));
					*fyptr++ = (float) (dy * atof(ys));
					break;
				}
				i++;
				if (i >= *count)
					break;
			}
			j++;
		}
		NEWLN(stream);
	} while (m != EOF);
	*count = i;
	if (mflag) {
		*x = realloc(*x, tsi * i);	/* Speicher wieder verkleinern */
		*y = realloc(*y, tsi * i);
	}
	if (fname[0] == '<')
		pclose(stream);
	else
		fclose(stream);
	return 0;
}

/************************************************************************************
*																					*
*    infodat:	liest Daten-Datei im SGotz-Binaerformat in Datenarrays.					*
*																					*   											 
*    EINGABEN:	x,y:  sind Pointer auf Pointer auf Datenarrays beliebigen	Typs	*
*				typ:  gibt den Typ der Datenarrays an:								*
*						"B"=Byte, "W"=2-Byte-Int, "L"=4-Byte-Int,					*
*						"F"=4-Byte-Float, "D"=8-Byte-Float, "E"=10-Byte-Float		*
*			  fname:  ist ein Pointer auf den Filenamen der Daten.					*
*    		  start:  gibt an ab dem wievieltem Datenpunkt eingelesen wird.			*
*						Es wird mit 0 begonnen zu zaehlen.							*
*			  count:  ist die Anzahl der einzulesenden Datenpunkte.					* 
*    		  dx,dy:  sind Normierungsfaktoren, mit denen die x bzw. y-				*
*					  Werte vor der Uebergabe multipliziert werden.					*
*    	xeinh,yeinh:  sind Pointer auf mind. 4-Byte grosse Chararrays				*
*    				  in denen die Einheitenstrings zurueckgegeben werden.			*
*    				  Dieser Strings sind nur dann richtig wenn dx=dy=1 sind.		*
*			  sinfo:  ist ein Pointer in dem das Sonderinfozeichen zurueck-			*
*					  geliefert wird, soweit vorhanden.								*
*			  siptr:  ist ein Pointer auf einen Speicherbereich, in dem				*
*					  die Sonderinformation, soweit vorhanden zurueck-				*
*					  geliefert wird. Ist siptr=NULL, so wird nichts				*
*					  zurueckgeliefert.												*
*			  silen:  ist ein Pointer in dem die Laenge der Sonderinfo				*
*					  zurueckgeliefert wird.										*	
*																					*   											 
*    ERGEBNIS:	ist 0 wenn alles ok,												*
*				-1 wenn falsches Dateiformat,										*
*				-2 wenn es Probleme mit dem oeffnen der Datei gibt					*
*				-3 wenn die Datei einen noch nicht implementierter Aufbau besitzt.	*
*				-4 bei zu wenig Speicher.											*
*************************************************************************************/
int infodat(char *fname, int *count, char *xeinh, char *yeinh, char *sinfo,
		  char *siptr, int *silen)
{
	int fehlnr = 0, dszahl = 0;
	int cnt0;
	double normx = 0.0, normy = 0.0, offx = 0.0, offy = 0.0, tleft, tright;
	
	FILE *stream;
	char kenn[6], dxtyp, dytyp, stuc;

	if (fname[0] == '<') {
		strcpy(xeinh, "arb");
		strcpy(yeinh, "arb");
		return 10;
	}
	
	stream = fopen(fname, "rb");	/* File oeffnen */
	if (stream != NULL) {
		fread(kenn, 1, 6l, stream);		/* Kennung einlesen */

		if (0 == strcmp("SGotz", kenn))
			fehlnr = rdkopf(stream, &tleft, &tright, &cnt0, &normx, &normy,
							xeinh, yeinh, sinfo, &dxtyp, &dytyp, &dszahl, &stuc,
							&offx, &offy);
		else if (0 == strcmp("SG2.0", kenn)) {
			fehlnr = rdkopfn(stream, &tleft, &tright, &cnt0, &normx, &normy,
							 xeinh, yeinh, sinfo, &dxtyp, &dytyp, &dszahl, &stuc,
							 &offx, &offy);
		} else {				/* gehe von Ascii Datenfile aus */
			fclose(stream);
			strcpy(xeinh, "arb");
			strcpy(yeinh, "arb");
			return 10;
		}

		if (fehlnr < 0)
			return fehlnr;

		if (*count == 0l)		/* Anzahl der Daten holen */
			*count = cnt0;
		else
			*count = MIN((cnt0), (*count));
		
		/* Sonderinfo einlesen */
		if (siptr != NULL && silen != NULL) {
			*silen = fread(siptr, 1l, 32768l, stream);
		}
		fclose(stream);			/* File schliesen */
	} else {
		fehlnr = -2;
	}
	
	return fehlnr;
}
/************************************************************************************
*																					*
*    radat:	liest Daten-Datei im SGotz-Binaerformat in Datenarrays.					*
*																					*   											 
*    EINGABEN:	x,y:  sind Pointer auf Pointer auf Datenarrays beliebigen	Typs	*
*				typ:  gibt den Typ der Datenarrays an:								*
*						"B"=Byte, "W"=2-Byte-Int, "L"=4-Byte-Int,					*
*						"F"=4-Byte-Float, "D"=8-Byte-Float, "E"=10-Byte-Float		*
*			  fname:  ist ein Pointer auf den Filenamen der Daten.					*
*    		  start:  gibt an ab dem wievieltem Datenpunkt eingelesen wird.			*
*						Es wird mit 0 begonnen zu zaehlen.							*
*			  count:  ist die Anzahl der einzulesenden Datenpunkte.					* 
*    		  dx,dy:  sind Normierungsfaktoren, mit denen die x bzw. y-				*
*					  Werte vor der Uebergabe multipliziert werden.					*
*    	xeinh,yeinh:  sind Pointer auf mind. 4-Byte grosse Chararrays				*
*    				  in denen die Einheitenstrings zurueckgegeben werden.			*
*    				  Dieser Strings sind nur dann richtig wenn dx=dy=1 sind.		*
*			  sinfo:  ist ein Pointer in dem das Sonderinfozeichen zurueck-			*
*					  geliefert wird, soweit vorhanden.								*
*			  siptr:  ist ein Pointer auf einen Speicherbereich, in dem				*
*					  die Sonderinformation, soweit vorhanden zurueck-				*
*					  geliefert wird. Ist siptr=NULL, so wird nichts				*
*					  zurueckgeliefert.												*
*			  silen:  ist ein Pointer in dem die Laenge der Sonderinfo				*
*					  zurueckgeliefert wird.										*	
*																					*   											 
*    ERGEBNIS:	ist 0 wenn alles ok,												*
*				-1 wenn falsches Dateiformat,										*
*				-2 wenn es Probleme mit dem oeffnen der Datei gibt					*
*				-3 wenn die Datei einen noch nicht implementierter Aufbau besitzt.	*
*				-4 bei zu wenig Speicher.											*
*************************************************************************************/
int radat(void **x, void **y, char typ, char *fname, int start, int *count,
		  double dx, double dy, char *xeinh, char *yeinh, char *sinfo,
		  char *siptr, int *silen)
{
	int fehlnr = 0, i, dszahl = 0;
	int tsi=0, yrsi=0, xrsi=0, cnt0, il;
	double normy = dy, normx = dx, offx = 0.0, offy = 0.0, tleft, tright;
	double x0, xd;
	double xn, xi;
	FILE *stream;
	double *dptr;
	float *fptr;
	int *lptr;
	short int *iptr;
	unsigned char *bptr;
	char kenn[6], dxtyp, dytyp, stuc;

	if (fname[0] == '<') {
		strcpy(xeinh, "arb");
		strcpy(yeinh, "arb");
		return rdascii(x, y, typ, fname, start, count, dx, dy, sinfo, siptr, silen);
	}
	
	stream = fopen(fname, "rb");	/* File oeffnen */
	if (stream != NULL) {
		fread(kenn, 1, 6l, stream);		/* Kennung einlesen */
		
		if (0 == strcmp("SGotz", kenn))
			fehlnr = rdkopf(stream, &tleft, &tright, &cnt0, &normx, &normy,
							xeinh, yeinh, sinfo, &dxtyp, &dytyp, &dszahl, &stuc,
							&offx, &offy);
		else if (0 == strcmp("SG2.0", kenn)) {
			fehlnr = rdkopfn(stream, &tleft, &tright, &cnt0, &normx, &normy,
							 xeinh, yeinh, sinfo, &dxtyp, &dytyp, &dszahl, &stuc,
							 &offx, &offy);
		} else {				/* gehe von Ascii Datenfile aus */
			fclose(stream);
			strcpy(xeinh, "arb");
			strcpy(yeinh, "arb");
			return rdascii(x, y, typ, fname, start, count, dx, dy ,sinfo, siptr, silen);
		}
				
		if (fehlnr < 0)
			return fehlnr;

		if (*count == 0l)		/* Anzahl der Daten holen */
			*count = cnt0 - start;
		else
			*count = MIN((cnt0 - start), (*count));
		
		switch (typ) {
		case 'b':
		case 'B':
			tsi = 1l;
			break;
		case 'W':
			tsi = 2l;
			break;
		case 'L':
		case 'F':
			tsi = 4l;
			break;
		case 'D':
			tsi = 8l;
			break;
		case 'E':
			tsi = 10l;
			break;
		}

		switch (dytyp) {
		case 'b':
		case 'B':
			yrsi = 1l;
			break;
		case 'W':
			yrsi = 2l;
			break;
		case 'L':
		case 'F':
			yrsi = 4l;
			break;
		case 'D':
			yrsi = 8l;
			break;
		case 'E':
			yrsi = 10l;
			break;
		}

		switch (dxtyp) {
		case 'b':
		case 'B':
			xrsi = 1l;
			break;
		case 'W':
			xrsi = 2l;
			break;
		case 'L':
		case 'F':
			xrsi = 4l;
			break;
		case 'D':
			xrsi = 8l;
			break;
		case 'E':
			xrsi = 10l;
			break;
		}

		if (*x == NULL) {		/* Speicher anfordern */
			*x = calloc(tsi, *count);
			if (*x == NULL) {	/* Zu wenig Speicher */
				fclose(stream);
				return -4;
			}
			*y = calloc(tsi, *count);
			if (*y == NULL) {	/* Zu wenig Speicher */
				free(*x);
				fclose(stream);
				return -4;
			}
		}
		dptr = (double*)*x;
		fptr = (float*)*x;
		lptr = (int*)*x;
		iptr = (short int*)*x;
		bptr = (unsigned char*)*x;
	
		/* X und Y-Array einlesen fuer den Fall */
		/* von Tupeldaten */
		if ((stuc == 'T') && (dszahl > 1)) {
			fseek(stream, (xrsi + yrsi) * start, SEEK_CUR);
			for (il = 0l; (il < *count) && (fehlnr == 0); il++) {
				fehlnr = rddat(stream, 1l, (char *) (*x) + il * tsi, typ, normx, offx, dxtyp);
				fehlnr = rddat(stream, 1l, (char *) (*y) + il * tsi, typ, normy, offy, dytyp);
			}
			fseek(stream, (xrsi + yrsi) * (cnt0 - (*count) - start), SEEK_CUR);

			goto w1;
		}
		
		/* X-Array einlesen/besetzen */
		if (dszahl > 1) {
			fseek(stream, xrsi * start, SEEK_CUR);
			fehlnr = rddat(stream, *count, *x, typ, normx, offx, dxtyp);
			fseek(stream, xrsi * (cnt0 - (*count) - start), SEEK_CUR);
		} else {				/* besetzen */
			x0 = tleft * normx;
			xn = tright * normx;
			/* printf("x0, xn: %f, %f\n",x0,xn); */
			xd = ((double) (xn - x0)) / (double) (cnt0 - 1l);
			xi = tleft * normx + (double) (start) * xd;
			for (i = 0; i < *count; i++) {
				switch (typ) {
				case 'D':
					*dptr++ = (double) xi;
					break;
				case 'F':
					*fptr++ = (float) xi;
					break;
				case 'L':
					*lptr++ = (long) xi;
					break;
				case 'W':
					*iptr++ = (int) xi;
					break;
				case 'b':
				case 'B':
					*bptr++ = (unsigned char) xi;
					break;
				}
				xi += xd;
			}
		}
		if (fehlnr != 0) {
			fclose(stream);
			return fehlnr;
		}
		
		/* y-Array einlesen */
		fseek(stream, yrsi * start, SEEK_CUR);
				
		fehlnr = rddat(stream, *count, *y, typ, normy, offy, dytyp);
		fseek(stream, yrsi * (cnt0 - (*count) - start), SEEK_CUR);

w1:		if (fehlnr != 0) {
			fclose(stream);
			return fehlnr;
		}
				
		/* Sonderinfo einlesen */
		if (siptr != NULL && silen != NULL) {
			*silen = fread(siptr, 1l, 32768l, stream);
		}
		fclose(stream);			/* File schliesen */
	} else {
		fehlnr = -2;
	}
	
	return fehlnr;
}

static int aadat(void *x, void *y, char xtyp, char ytyp, int start, int count,
				 double dx, double dy, double offx, double offy, char *xeinh, char *yeinh,
				 double xbegin, double xend, char soinf, void *siptr, int silen,
				 FILE * stream)
{
	VORSP vs;
	DASATZ dsx, dsy;
	int ytlen, xtlen = 0l;
	int fehlnr = 0;
	double ddum;
	unsigned char data[1280], *cp, *dp;
	int i, j, k;

	fread(&vs, 1, sizeof(VORSP), stream);	/* Vorspann einlesen */
	chabyte(&(vs.count), 4);
	chabyte(&(vs.datzahl), 2);
	chabyte(&(vs.tleft), 8);
	chabyte(&(vs.tright), 8);

	if (0 != strcmp("SG2.0", vs.kennung)) {
		return -1;
	}
	if ((vs.datzahl > 1) && (vs.stuc != 'T') && (x != y))
		return -3;
	vs.count += count;

	if (siptr != NULL)
		vs.sinfo = soinf;

	if (vs.datzahl > 1) {
		if (xbegin != xend)
			return -4;
		fread(&dsx, 1, sizeof(DASATZ), stream);		/* X-DASATZ einlesen */
		chabyte(&(dsx.offset), 8);
		chabyte(&(dsx.norm), 8);
		if (dsx.dtyp != xtyp)
			return -4;
		ddum = dsx.offset;
		if (ddum != offx)
			return -4;
		ddum = dsx.norm;
		if (ddum != dx)
			return -4;;
		if (0 != strcmp(dsx.einh, xeinh))
			return -4;
	}
	fread(&dsy, 1, sizeof(DASATZ), stream);		/* Y-DASATZ einlesen */
	chabyte(&(dsy.offset), 8);
	chabyte(&(dsy.norm), 8);
	if (dsy.dtyp != ytyp)
		return -4;
	ddum = dsy.offset;
	if (ddum != offy)
		return -4;
	ddum = dsy.norm;
	if (ddum != dy)
		return -4;
	if (0 != strcmp(dsy.einh, yeinh)) {
		return -4;
	}
	if (xbegin != xend)			/* rechte X-Grenze aktualisieren */
		vs.tright = xend;
	else if (vs.datzahl == 1)
		return -4;

	fseek(stream, 0, 0);
	chabyte(&(vs.count), 4);
	chabyte(&(vs.datzahl), 2);
	chabyte(&(vs.tleft), 8);
	chabyte(&(vs.tright), 8);
	fwrite(&vs, 1, sizeof(VORSP), stream);	/* Vorspann abspeichern */

	switch (ytyp) {				/* y-Elementlaenge bestimmen */
	case 'b':
	case 'B':
		ytlen = 1l;
		break;
	case 'W':
		ytlen = 2l;
		break;
	case 'L':
	case 'F':
		ytlen = 4l;
		break;
	case 'D':
		ytlen = 8l;
		break;
	case 'E':
		ytlen = 10l;
		break;
	default:
		return -5;
	}
	if (vs.stuc == 'T') {
		switch (xtyp) {			/* x-Elementlaenge bestimmen */
		case 'b':
		case 'B':
			xtlen = 1l;
			break;
		case 'W':
			xtlen = 2l;
			break;
		case 'L':
		case 'F':
			xtlen = 4l;
			break;
		case 'D':
			xtlen = 8l;
			break;
		case 'E':
			xtlen = 10l;
			break;
		default:
			return -5;
		}
	}
	fseek(stream, 0, 2);		/* Zum Dateiende */
	/* Daten speichern */
	dp = (unsigned char *) y + start * (ytlen + xtlen);
	for (i = 0; i < count * (ytlen + xtlen);) {		/* xy-Daten speichern */
		for (j = 0; (j < 640) && (i < count * (ytlen + xtlen)); j++, i++)
			data[j] = *dp++;
		for (k = 0, cp = data; (k < j); k += (xtlen + ytlen), cp += (xtlen + ytlen)) {
			chabyte(cp, xtlen);
			chabyte(cp + xtlen, ytlen);
		}
		fwrite(data, 1, j, stream);
	}
	if (soinf != 0 && siptr != NULL) {	/* Sonderinfo abspeichern */
		fwrite(siptr, 1, silen, stream);
	}
	return fehlnr;

}

/*****************************************************************************
*    				    					     *		
*    wadat:		schreibt Daten im SG2.0-Binaerformat in eine Datei.  *
*    				    					     *							*
*    EINGABEN:	x,y:	sind Pointer auf Datenarrays beliebigen Typs.	     *
*			Zeigen x und y auf den selben Speicherbereich, so    *
*			wird davon ausgegeangen, dass die Daten in einem     *
*			Tupel-Array vorliegen (x0,y0)(x1,y1)... .	Die  *
*			Daten werden dann als Tupel (x,y) abgespeichert, was *
*			beim Lesen der Daten viel Zeit kostet. Es koennen    *
*			aber an solche Dateien Daten angehaengt werden.	     * 
*		xtyp,ytyp:gibt den Typ der Datenarrays an: "B" =	Byte *
*			"W"=2-Byte-Int,"L"=4-Byte-Int,"F"=4-Byte-Float	     *
*			"D"=8-Byte-Float,"E"=10-Byte-Float		     *
*		fname:	ist ein Pointer auf den Filenamen der Daten.	     *
*    		start:	gibt an ab dem wievieltem Arrayindex abgespeichert   *
*			wird.Es wird mit 0 begonnen zu zaehlen.		     *
*		count:  ist die Anzahl der abzuspeichernden Datenpunkte.     * 
*    		dx,dy:  sind Normierungsfaktoren, die im Dateikopf    	     *
*    		       	gespeichert werden, und mit denen die Werte vor dem  *
*			lesen multipliziert werden.	    		     *
*		offx,offy: sind die Offsets. die im Dateikopf gespeichert    *
*			werden. Diese Offsets werden beim lesen der Daten    *
*			nach dem multiplizieren mit den Normierungsfaktoren  *
*			aufaddiert.					     *
* 		xeinh,yeinh: sind Pointer auf mind. 4-Byte grosse Chararrays *
*    			in denen die Einheitenstrings uebergeben werden.     *
*			xbegin,xend: sind bei aequidistanten Daten in X die  * 
*    			linke u. rechte X-Grenze. Das X-Array wird bei       *
*    			xbegin!=xend nicht abgespeichert, da es beim lesen   *
*    	 		aus xbegin und xend rekunstruiert wird.	   	     *
*		sinfo: 	ist ein Pointer in dem das Sonderinfozeichen ueber-  *
*			geben wird.					     *
*		siptr: 	ist ein Pointer auf einen Speicherbereich, in dem    *
*			die Sonderinformation, uebergeben wird. Ist siptr=   *
*			NULL, so wird nichts diese nicht abgespeichert.      *
*		silen: 	ist ein Pointer in dem die Laenge der Sonderinfo     *
*			uebergeben wird.				     *
*		cont:	wenn cont !=0 ist, so wird an eine besehende Datei   *
*			angehaengt. Es koennen nur Tupel-Daten an ebensolche *
*			Dateien angehaengt werden, mit Ausnahme von Daten    *
*			mit aequidistanten Punkten. Dabei mussen jedoch die  *
*			angefuegten Daten ebenfalls aequidistant zu den      *
*			Daten in der Datei sein.			     *
*    ERGEBNIS:	ist 0 wenn alles ok und -2 wenn es Probleme mit dem oeffnen  *
*				der Datei gibt.			     	     *
*    				    					     *
****************************************************************************/
int wadat(void *x, void *y, char xtyp, char ytyp, char *fname, int start, int count,
		  double dx, double dy, double offx, double offy, char *xeinh, char *yeinh,
		  double xbegin, double xend, char soinf, char *siptr, int silen, int cont)
{
	int fehlnr = 0;
	int xtlen, ytlen;
	VORSP vs;
	DASATZ dsx;
	DASATZ dsy;
	FILE *stream;
	unsigned char data[1280], *cp, *dp;
	int i, j, k;

	strcpy(vs.kennung, "SG2.0");
	strcpy(vs.teinh, xeinh);
	vs.datzahl = 2;
	if (x == y)					/* => Tupel */
		vs.stuc = 'T';
	else						/* => Reihe */
		vs.stuc = 'R';

	vs.sinfo = soinf;
	vs.count = count;
	vs.tleft = xbegin * dx + offx;
	vs.tright = xend * dx + offx;

	dsy.dtyp = ytyp;
	dsy.res = 0;
	strcpy(dsy.einh, yeinh);
	dsy.norm = dy;
	dsy.offset = offy;

	if (xbegin != xend) {		/* kann x-Array ignoriert werden?   */
		vs.datzahl = 1;			/* => ja */
	} else {
		dsx.dtyp = xtyp;
		dsx.res = 0;
		strcpy(dsx.einh, xeinh);
		dsx.norm = dx;
		dsx.offset = offx;
	}

	if (cont) {
		stream = fopen(fname, "r+");	/* File oeffnen */
		if (stream != NULL) {
			fehlnr = aadat(x, y, xtyp, ytyp, start, count, dx, dy, offx, offy,
						   xeinh, yeinh, xbegin, xend, soinf, siptr,
						   silen, stream);
			fclose(stream);
			return fehlnr;
		}
	}
	stream = fopen(fname, "wb");	/* File oeffnen */
	if (stream != NULL) {
		chabyte(&(vs.count), 4);
		chabyte(&(vs.datzahl), 2);
		chabyte(&(vs.tleft), 8);
		chabyte(&(vs.tright), 8);
		fwrite(&vs, 1, sizeof(VORSP), stream);	/* Vorspann abspeichern */
		if (xbegin == xend) {	/* x-Datensatz speichern */
			chabyte(&(dsx.offset), 8);
			chabyte(&(dsx.norm), 8);
			fwrite(&dsx, 1, sizeof(DASATZ), stream);
		} else {
		}
		/* y-Datensatz speichern */
		chabyte(&(dsy.offset), 8);
		chabyte(&(dsy.norm), 8);
		fwrite(&dsy, 1, sizeof(DASATZ), stream);

		switch (xtyp) {			/* Elementlaenge bestimmen */
		case 'b':
		case 'B':
			xtlen = 1l;
			break;
		case 'W':
			xtlen = 2l;
			break;
		case 'L':
		case 'F':
			xtlen = 4l;
			break;
		case 'D':
			xtlen = 8l;
			break;
		case 'E':
			xtlen = 10l;
			break;
		default:
			return -5;
		}
		switch (ytyp) {			/* Elementlaenge bestimmen */
		case 'b':
		case 'B':
			ytlen = 1l;
			break;
		case 'W':
			ytlen = 2l;
			break;
		case 'L':
		case 'F':
			ytlen = 4l;
			break;
		case 'D':
			ytlen = 8l;
			break;
		case 'E':
			ytlen = 10l;
			break;
		default:
			return -5;
		}

		if (vs.stuc == 'R') {
			if (xbegin == xend) {	/* x-Daten speichern */
				dp = (unsigned char *) x + start * xtlen;
				for (i = 0; i < count * xtlen;) {	/* x-Daten speichern */
					for (j = 0; (j < 640) && (i < count * xtlen);
						 j++, i++)
						data[j] = *dp++;
					for (k = 0, cp = data; (k < j); k += xtlen,
						 cp += xtlen)
						chabyte(cp, xtlen);
					fwrite(data, 1, j, stream);
				}
			}
			dp = (unsigned char *) y + start * ytlen;
			for (i = 0; i < count * ytlen;) {	/* y-Daten speichern */
				for (j = 0; (j < 640) && (i < count * ytlen); j++, i++)
					data[j] = *dp++;
				for (k = 0, cp = data; (k < j); k += ytlen, cp += ytlen)
					chabyte(cp, ytlen);
				fwrite(data, 1, j, stream);
			}					/* y-Daten speichern */
		} else {				/* x-y-Daten speichern */
			dp = (unsigned char *) y + start * (ytlen + xtlen);
			for (i = 0; i < count * (ytlen + xtlen);) {		/* xy-Daten speichern */
				for (j = 0; (j < 640) && (i < count * (ytlen + xtlen));
					 j++, i++)
					data[j] = *dp++;
				for (k = 0, cp = data; (k < j); k += (xtlen + ytlen),
					 cp += (xtlen + ytlen)) {
					chabyte(cp, xtlen);
					chabyte(cp + xtlen, ytlen);
				}
				fwrite(data, 1, j, stream);
			}
		}

		if (soinf != 0 && siptr != NULL) {	/* Sonderinfo abspeichern */
			fwrite(siptr, 1, silen, stream);
		}
		fclose(stream);
	} else {
		fehlnr = -2;
	}
	return fehlnr;
}

static char siptr[32768];
char buf[32768];

/****************************************************************************
*																			*
*    infocalc: analysiert die Sonderinfo an jedem Lecroyfile				*
*																			*
*    EINGABEN: siptr = Pointer auf den Sonderinfostring						*
*    	       silen = Laenge der Sonderinfo								*
*    	       fptr  = Pointer auf ein 50 Elemente float-Array				*
*    	       iptr  = Pointer auf ein 50 Elemente shortint-Array			*
*    	       cptr  = Pointer auf ein 50 Elemente char-Array				*
*    ERGEBNIS: fptr: 1-10 :Pulsersequenz (h l h l ...)						*
*    	             11   :Frequenz [Hz]									*
*    	    	     12   :dB[dB]											*
*					 13-17:Lecroy (TB[Hz] VD[V] TRD[s] TRC LBSC)			*
*					 18-19:Matek  (Gain,Tune)								*
*					 20-27:PLMstatus										*
*			   iptr: 1	  :Muxbit											*
*					 2-6  :Datum (Tag,Monat,Jahr,Stunde,Minute)				*
*    	       cptr: 1-?  :pfad  (mit 0 abgeschlossen)						*
*    	       Ist ein Eintrag auf -1 gesetzt, so ist keine Info dazu da.   * 
*																			*
****************************************************************************/
void infocalc(char *siptr, int silen, float *fptr, short int *iptr, char *cptr)
{
	int i, pi = 0, pf = 0, i1, i2, i3, i4, i5, i6;
	char *sp, *sp1, ch1, ch0, ch2, ar[100], ar2[100];
	char ss[10][20];

	pi = 0;
	pf = 0;
	siptr[silen] = 0;			/* sicher ist sicher */
	sp = strstr(siptr, "pulser.s");		/* Pulserstatus suchen */
	if (sp != NULL) {			/* und auswerten */
		sp = strstr(sp, "\n");
		sp++;
		ch1 = 'h';
		ch2 = 'l';
		do {
			if (*sp == ch1) {
				sp++;
				fptr[pf++] = atof(sp);
				ch0 = ch1;
				ch1 = ch2;
				ch2 = ch0;
			}
			sp++;
		} while ((*sp) != '\n' && (*sp) != 0 && pf < 10);
	}
	for (; pf < 10; pf++)
		fptr[pf] = -1;

	sp = strstr(siptr, "rsgo");	/* Generatorstatus suchen */
	if (sp != NULL) {			/* und auswerten */
		sp = strstr(sp, "\n");
		sp++;
		fptr[pf++] = atof(sp);
	} else {
		fptr[pf++] = -1;
	}

	sp = strstr(siptr, "rsat");	/* Abschwaecherstatus suchen */
	if (sp != NULL) {			/* und auswerten */
		sp = strstr(sp, "\n");
		sp++;
		fptr[pf++] = atof(sp);
	} else {
		fptr[pf++] = -1;
	}

	sp = strstr(siptr, "TB=");	/* Timebase suchen */
	if (sp != NULL) {			/* und auswerten */
		sp = sp + 3;
		fptr[pf++] = atof(sp) / 1.0E6;
	} else {
		fptr[pf++] = -1;
	}
	sp = strstr(siptr, "VD=");	/* VD suchen */
	if (sp != NULL) {			/* und auswerten */
		sp = sp + 3;
		fptr[pf++] = atof(sp) / 1.0E3;
	} else {
		fptr[pf++] = -1;
	}
	sp = strstr(siptr, "TRD=");	/* TRD suchen */
	if (sp != NULL) {			/* und auswerten */
		sp = sp + 4;
		fptr[pf++] = atof(sp);
	} else {
		fptr[pf++] = -1;
	}
	sp = strstr(siptr, "TRC=");	/* TRC suchen */
	if (sp != NULL) {			/* und auswerten */
		sp = sp + 4;
		fptr[pf++] = atof(sp);
	} else {
		fptr[pf++] = -1;
	}
	sp = strstr(siptr, "LBSC=");	/* LBSC suchen */
	if (sp != NULL) {			/* und auswerten */
		sp = sp + 5;
		fptr[pf++] = atof(sp);
	} else {
		fptr[pf++] = -1;
	}

	sp = strstr(siptr, "matek");	/* Matekstatus suchen */
	if (sp != NULL) {			/* und auswerten */
		sp = strstr(sp, "\n");
		sp++;
		sp1 = strstr(sp, "gain");
		sp++;
		if (sp1 != NULL)
			fptr[pf++] = atof(sp1 + 4);
		else
			fptr[pf++] = -1;
		sp1 = strstr(sp, "tune");
		sp++;
		if (sp1 != NULL)
			fptr[pf++] = atof(sp1 + 4);
		else
			fptr[pf++] = -1;
	} else {
		fptr[pf++] = -1;
	}

	sp = strstr(siptr, "plm.s");	/* plmstatus suchen */
	if (sp != NULL) {			/* und auswerten */
		sp = strstr(sp, "\n");
		sp++;
		sscanf(sp, "%s %s %s %s %s %s %s %s %s",
			   ss[0], ss[1], ss[2], ss[3], ss[4], ss[5], ss[6], ss[7], ss[8]);
		for (i = 1; i < 9; i++) {
			if (ss[i][0] != ',')
				fptr[pf++] = atof(ss[i]);
			else
				fptr[pf++] = -1.0;
		}
	} else {
		fptr[pf++] = -1;
	}

	sp = strstr(siptr, "mux.s");	/* Muxbitstatus suchen */
	if (sp != NULL) {			/* und auswerten */
		sp = strstr(sp, "&");
		sp++;
		iptr[pi] = 16 * 16 * 16 * (short) strtol(sp++, &sp, 2);
		sp++;
		iptr[pi] += 16 * 16 * (short) strtol(sp++, &sp, 2);
		sp++;
		iptr[pi] += 16 * (short) strtol(sp++, &sp, 2);
		sp++;
		iptr[pi++] += (short) strtol(sp, &sp, 2);
	} else {
		iptr[pi++] = -1;
	}
	sp = strstr(siptr, "pfad:");	/* pfad suchen */
	if (sp != NULL) {			/* und auswerten */
		sp = strstr(sp, "\n");
		sp++;
		for (i = 0;; i++) {
			if (*sp != ' ' && *sp != 0 && *sp != '\n')
				cptr[i] = *sp++;
			else {
				i++;
				cptr[i] = 0;
				break;
			}
		}
	} else {
		*cptr = 0;
	}
	sp = strstr(siptr, "date:");	/* datum suchen */
	if (sp != NULL) {			/* und auswerten */
		sp = strstr(sp, "\n");
		sp++;
		sscanf(sp, "%d %s %d %d:%d:%d", &i1, ar, &i2, &i3, &i4, &i5);
		strcpy(ar2, "JanFebMarAprMayJunJulAugSepNovDec");
		sp1 = strstr(ar2, ar);
		i6 = (int) ((ar2 - sp1) / 3) + 1;
		iptr[pi++] = i1;
		iptr[pi++] = i6;
		iptr[pi++] = i2;
		iptr[pi++] = i3;
		iptr[pi++] = i4;
		iptr[pi++] = i5;
	} else {
		iptr[pi++] = -1;
		iptr[pi++] = -1;
		iptr[pi++] = -1;
		iptr[pi++] = -1;
		iptr[pi++] = -1;
		iptr[pi++] = -1;
	}

	sp = strstr(siptr, "GE:");	/* Girgl Eintraege suchen */
	if (sp != NULL) {			/* und auswerten */
		sp = strstr(sp, "\n");
		sp++;
		strcpy(buf, sp);
		sp1 = strtok(buf, " ");
		for (i = 0; i < 50; i++) {
			if (sp1 != NULL) {
				fptr[i] = atof(sp1);
				sp1 = strtok(NULL, " \r\n");
			} else
				fptr[i] = -1.0;
		}
		for (i = 0; i < 50; i++) {
			if (sp1 != NULL) {
				iptr[i] = (short int) atoi(sp1);
				sp1 = strtok(NULL, " \r\n");
			} else
				iptr[i] = (short int) (-1);
		}
		strcpy(cptr, sp1);
	}
}

int fradat_(void *x, void *y, char *fname, int *start, int *count,
			double *dx, double *dy, char *xeinh, char *yeinh,
			float *fptr, short int *iptr, char *cptr, char *dataformat, char *sinfo)
{
	int silen, ret;
	
	if (*start > 0) (*start)--;
	
	ret = radat(&x, &y, *dataformat, fname, *start, count, *dx, *dy, xeinh, yeinh, sinfo, siptr, &silen);
	
	if (ret < 0) {
		printf("Fehler bei radat(): Returnwert=%i\n", ret);
	}
	
	/* infocalc(siptr, silen, fptr, iptr, cptr); */

	return ret;
}

int fwadat_(float *x, float *y, char *fname, int *start, int *count,
			double *dx, double *dy, char *xeinh, char *yeinh, double *xbegin,
			double *xend, float *fptr, short int *iptr, char *cptr,
			char *dataformat, char *soinf)
{
	int i, silen = 0;
	char buf[200];

	if (*start > 0) (*start)--;

	strcpy(siptr, "GE:\n");
	for (i = 0; i < 50; i++) {
		sprintf(buf, "% 8e ", fptr[i]);
		strcat(siptr, buf);
		if (i % 5 == 4)
			strcat(siptr, "\n");
	}
	for (i = 0; i < 50; i++) {
		sprintf(buf, "% 6hi ", iptr[i]);
		strcat(siptr, buf);
		if (i % 10 == 9)
			strcat(siptr, "\n");
	}
	strncat(siptr, cptr, 50);
	strcat(siptr, "\n");
	silen = (int) strlen(siptr);

	return wadat(x, y, *dataformat, *dataformat, fname, *start, *count, *dx, *dy, 0.0, 0.0, xeinh, yeinh,
				 *xbegin, *xend, *soinf, siptr, silen, 0);
}
