
/****************************************************************************
*    				    					    							*
*    DATEN-Modul von Siegfried Goetz					    				*
*    				    					    							*
*    letzte Aenderung 27.07.92						    					*
*    				    					    							*
****************************************************************************/

/* Folgende Struktur ist allen Daten im neuen "SG2.0"-Format vorangestellt. */

typedef struct
{
     	char	kennung[6];	/* Formatkennung = "SG2.0" */
	char	teinh[4];	/* Einheit der Zeitachse */
	char	stuc;	 	/* Datenstruktur : 'R'=Datenreihen => Datensaetze sind hintereinander abgelegt 	*/ 
						/*				   'T'=Datentupel  => Datensaetze sind nebeneinander abgelegt	*/
	char	sinfo;		/* Sonderinfokennzeichen : Sonderinfo wird nach den Daten angehaengt			*/
	int	count;		/* Anzahl der Datenpunkte	*/
	short   datzahl;	/* Anzahl der folgenden Datensaetze	*/
	char    res[6];         /* reserviert */
	double	tleft;		/* linke und rechte Zeitgrenze der Daten wenn tleft=tright=0  */
	double	tright;		/*	=> kein solch Interpretation moeglich */
}VORSP;

typedef struct
{
	double	offset;	 	/* Offset der Daten	*/
	double	norm;	 	/* Normierungsfaktor der Daten bzgl Einheit	*/
	char 	dtyp;	 	/* Datentype 	 : 'B'=Byte, 'W'=Word, 'L'=Long, 'F'=Float, 'D'=Doublefloat 	*/
	char	res;		/* reserviertes Byte	*/						
	char	einh[6];	/* Einheit der Daten */
}DASATZ;

/*   DATEN
		.
       	.
       	.
*/
/*
	Sonderinformation
		.
		.
		.	
*/

/* Folgende Struktur ist allen Daten im "SGotz" vorangestellt. */


typedef struct
{
    char	kennung[6];	/* Formatkennung = "SGotz" */
	short int datzahl;	/* Anzahl der folgenden Datensaetze	*/	
	char 	dtyp;	 	/* Datentype 	 : 'B'=Byte, 'W'=Word, 'L'=Long, 'F'=Float, 'D'=Doublefloat 	*/
	char	stuc;	 	/* Datenstruktur : 'R'=Datenreihen => Datensaetze sind hintereinander abgelegt 	*/ 
						/*				   'T'=Datentupel  => Datensaetze sind nebeneinander abgelegt	*/
	char	sinfo;		/* Sonderinfokennzeichen : Sonderinfo wird nach den Daten angehaengt			*/
	char	res;		/* reserviertes Byte	*/						
}VORSPANN;

typedef struct
{
	float	norm;	 	/* Normierungsfaktor der Daten bzgl Einheit	*/
	int	count;		/* Anzahl der Daten	*/
	char	einh[4];	/* Einheit der Daten */
	float	tleft;		/* linke und rechte Zeitgrenze der Daten wenn tleft=tright=0  */
	float	tright;		/*	=> kein solch Interpretation moeglich */
	char	teinh[4];	/* Einheit der Zeitachse */
}DATENSATZ;

/*   DATEN
		.
       	.
       	.
*/
/*
	Sonderinformation
		.
		.
		.	
*/


/*
Alle float Werte sind nach folgender Norm zu interpretieren:

    nach IEEE   | Pure C Syntax   | Wertebereich von bis | Genauigkeit
----------------+-----------------+----------------------+------------
single real     | float           |   10E-38 ..   10E+38 | 32 Bit

*/
/* Ein Datensatz vom Lecroy z.B.:

	"SGotz"
	1
	'B'
	'R'
	''
	''

	
	1.0 E-6	
	32001
	"V"
	0.0
	50.0 E-3
	"sec"
	
	200
	201
	202
	203
	203
	202
	.
	.
	.
	
*/
#define MIN(a,b)  (((a) < (b)) ? (a) : (b)) 
#define MAX(a,b)  (((a) > (b)) ? (a) : (b))


/****************************************************************************
*    				    					    							*
*    chabyte:  vertauscht die Bytereihenfolge eines Speicherbereichs.	    *
*    				    					    							*
*    EINGABEN: ptr ist ein Pointer auf den Speicherbereich.		    		*
*    	       anz ist die Anzahl der Bytes die vertauscht werden soll.	    *
*    ERGEBNIS: keines.							    						*
*    				    					    							*   											  
****************************************************************************/
void chabyte(void *ptr,long anz);

/*****************************************************************************
*                                                                            *
*    infodat:     liest Daten-Datei im SGotz-Binaerformat in Datenarrays.      *
*                                                                            *
*    EINGABEN:  x,y:    sind Pointer auf Pointer auf Datenarrays beliebigen  *
*                       Typs                                                 *
*               typ:    gibt den Typ der Datenarrays an: "B" =  Byte         *
*                       "W"=2-Byte-Int,"L"=4-Byte-Int,"F"=4-Byte-Float       *
*                       "D"=8-Byte-Float,"E"=10-Byte-Float                   *
*               fname:  ist ein Pointer auf den Filenamen der Daten.         *
*               start:  gibt an ab dem wievieltem Datenpunkt eingelesen wird *
*                       Es wird mit 0 begonnen zu zaehlen.                   *
*               count:  ist die Anzahl der einzulesenden Datenpunkte.        *
*               dx,dy:  sind Normierungsfaktoren, mit denen die x bzw. y-    *
*                       Werte vor der Uebergabe multipliziert werden.        *
*               xeinh,yeinh: sind Pointer auf mind. 4-Byte grosse Chararrays *
*                       in denen die Einheitenstrings zurueckgegeben werden. *
*                       Dieser Strings sind nur dann richtig wenn dx=dy=1    *
*                       sind.                                                *
*               sinfo:  ist ein Pointer in dem das Sonderinfozeichen zurueck *
*                       -geliefert wird, soweit vorhanden.                   *
*               siptr:  ist ein Pointer auf einen Speicherbereich, in dem    *
*                       die Sonderinformation, soweit vorhanden zurueck-     *
*                       geliefert wird. Ist siptr=NULL, so wird nichts       *
*                       zurueckgekiefert.                                    *
*                       silen:  ist ein Pointer in dem die Laenge der Sonder-*
*                       info zurueckgeliefert wird.                          *
*    ERGEBNIS:  ist 0 wenn alles ok, -1 wenn falsches Dateiformat,-2 wenn    *
*               es Probleme mit dem oeffnen der Datei gibt und -3 wenn die   *
*               Datei einen noch nicht implementierter Aufbau besitzt.Und -4 *
*               bei zu wenig Speicher.                                       *
*                                                                            *
******************************************************************************/
int infodat(char *fname, int *count, char *xeinh, char *yeinh, char *sinfo,
                char *siptr, int *silen);
/*****************************************************************************
*                                                                            *
*    radat:     liest Daten-Datei im SGotz-Binaerformat in Datenarrays.      *
*                                                                            *
*    EINGABEN:  x,y:    sind Pointer auf Pointer auf Datenarrays beliebigen  *
*                       Typs                                                 *
*               typ:    gibt den Typ der Datenarrays an: "B" =  Byte         *
*                       "W"=2-Byte-Int,"L"=4-Byte-Int,"F"=4-Byte-Float       *
*                       "D"=8-Byte-Float,"E"=10-Byte-Float                   *
*               fname:  ist ein Pointer auf den Filenamen der Daten.         *
*               start:  gibt an ab dem wievieltem Datenpunkt eingelesen wird *
*                       Es wird mit 0 begonnen zu zaehlen.                   *
*               count:  ist die Anzahl der einzulesenden Datenpunkte.        *
*               dx,dy:  sind Normierungsfaktoren, mit denen die x bzw. y-    *
*                       Werte vor der Uebergabe multipliziert werden.        *
*               xeinh,yeinh: sind Pointer auf mind. 4-Byte grosse Chararrays *
*                       in denen die Einheitenstrings zurueckgegeben werden. *
*                       Dieser Strings sind nur dann richtig wenn dx=dy=1    *
*                       sind.                                                *
*               sinfo:  ist ein Pointer in dem das Sonderinfozeichen zurueck *
*                       -geliefert wird, soweit vorhanden.                   *
*               siptr:  ist ein Pointer auf einen Speicherbereich, in dem    *
*                       die Sonderinformation, soweit vorhanden zurueck-     *
*                       geliefert wird. Ist siptr=NULL, so wird nichts       *
*                       zurueckgekiefert.                                    *
*                       silen:  ist ein Pointer in dem die Laenge der Sonder-*
*                       info zurueckgeliefert wird.                          *
*    ERGEBNIS:  ist 0 wenn alles ok, -1 wenn falsches Dateiformat,-2 wenn    *
*               es Probleme mit dem oeffnen der Datei gibt und -3 wenn die   *
*               Datei einen noch nicht implementierter Aufbau besitzt.Und -4 *
*               bei zu wenig Speicher.                                       *
*                                                                            *
******************************************************************************/
int radat(void **x,void **y,char typ,char *fname,int start,int *count,
                double dx, double dy,char *xeinh, char *yeinh, char *sinfo,
                char *siptr, int *silen);
/*****************************************************************************
*                                                                            *
*    wadat:             schreibt Daten im SG2.0-Binaerformat in eine Datei.  *
*                                                                            *
*    EINGABEN:  x,y:    sind Pointer auf Datenarrays beliebigen Typs.        *
*                       Zeigen x und y auf den selben Speicherbereich, so    *
*                       wird davon ausgegeangen, dass die Daten in einem     *
*                       Tupel-Array vorliegen (x0,y0)(x1,y1)... .       Die  *
*                       Daten werden dann als Tupel (x,y) abgespeichert, was *
*                       beim Lesen der Daten viel Zeit kostet. Es koennen    *
*                       aber an solche Dateien Daten angehaengt werden.      *
*               xtyp,ytyp:gibt den Typ der Datenarrays an: "B" =        Byte *
*                       "W"=2-Byte-Int,"L"=4-Byte-Int,"F"=4-Byte-Float       *
*                       "D"=8-Byte-Float,"E"=10-Byte-Float                   *
*               fname:  ist ein Pointer auf den Filenamen der Daten.         *
*               start:  gibt an ab dem wievieltem Arrayindex abgespeichert   *
*                       wird.Es wird mit 0 begonnen zu zaehlen.              *
*               count:  ist die Anzahl der abzuspeichernden Datenpunkte.     *
*               dx,dy:  sind Normierungsfaktoren, die im Dateikopf           *
*                       gespeichert werden, und mit denen die Werte vor dem  *
*                       lesen multipliziert werden.                          *
*               offx,offy: sind die Offsets. die im Dateikopf gespeichert    *
*                       werden. Diese Offsets werden beim lesen der Daten    *
*                       nach dem multiplizieren mit den Normierungsfaktoren  *
*                       aufaddiert.                                          *
*               xeinh,yeinh: sind Pointer auf mind. 4-Byte grosse Chararrays *
*                       in denen die Einheitenstrings uebergeben werden.     *
*                       xbegin,xend: sind bei aequidistanten Daten in X die  *
*                       linke u. rechte X-Grenze. Das X-Array wird bei       *
*                       xbegin!=xend nicht abgespeichert, da es beim lesen   *
*                       aus xbegin und xend rekunstruiert wird.              *
*               sinfo:  ist ein Pointer in dem das Sonderinfozeichen ueber-  *
*                       geben wird.                                          *
*               siptr:  ist ein Pointer auf einen Speicherbereich, in dem    *
*                       die Sonderinformation, uebergeben wird. Ist siptr=   *
*                       NULL, so wird nichts diese nicht abgespeichert.      *
*               silen:  ist ein Pointer in dem die Laenge der Sonderinfo     *
*                       uebergeben wird.                                     *
*               cont:   wenn cont !=0 ist, so wird an eine besehende Datei   *
*                       angehaengt. Es koennen nur Tupel-Daten an ebensolche *
*                       Dateien angehaengt werden, mit Ausnahme von Daten    *
*                       mit aequidistanten Punkten. Dabei mussen jedoch die  *
*                       angefuegten Daten ebenfalls aequidistant zu den      *
*                       Daten in der Datei sein.                             *
*    ERGEBNIS:  ist 0 wenn alles ok und -2 wenn es Probleme mit dem oeffnen  *
*                               der Datei gibt.                              *
*                                                                            *
****************************************************************************/
int wadat(void *x,void *y,char xtyp,char ytyp, char *fname,int start,int count,
          double dx, double dy,double offx, double offy,char *xeinh,char *yeinh,
          double xbegin,double xend,char soinf,char *siptr,int silen,int cont);
                                                                                
/****************************************************************************
*    				    					    *
*    infocalc: analysiert die Sonderinfo an jedem Lecroyfile	    	    *
*    EINGABEN: siptr = Pointer auf den Sonderinfostring	    		    *
*    	       silen = Laenge der Sonderinfo			 	    *
*    	       fptr  = Pointer auf ein 50 Elemente float-Array		    *
*    	       iptr  = Pointer auf ein 50 Elemente shortint-Array	    *
*    	       cptr  = Pointer auf ein 50 Elemente char-Array		    *
*    ERGEBNIS: fptr: 1-10 :Pulsersequenz (h l h l ...)	 	    	    *
*    	             11   :Frequenz [Hz]				    *
*    	    	     12   :dB[dB]					    *
*    		     13-17:Lecroy (TB[Hz] VD[V] TRD[s] TRC LBSC)	    *
*    		     18-19:Matek  (Gain,Tune)				    *
*    	       iptr: 1	  :Muxbit					    *
*    		     2-6  :Datum (Tag,Monat,Jahr,Stunde,Minute)		    *
*    	       cptr: 1-?  :pfad  (mit 0 abgeschlossen)			    *
*    	       Ist ein Eintrag auf -1 gesetzt, so ist keine Info dazu da.   * 
*    				    				   	    *
****************************************************************************/
void infocalc(char *siptr,int silen,float *fptr,short int *iptr,char *cptr);

int fradat_(void *x,void *y,char *fname,int *start,int *count,
          double *dx, double *dy, char *xeinh, char *yeinh,
          float *fptr,short int *iptr,char *cptr,char *typ,char *sinfo);

int fwadat_(float *x,float *y,char *fname,int *start,int *count,
            double *dx,double *dy,char *xeinh, char *yeinh,double *xbegin,
            double *xend,float *fptr,short int *iptr,char *cptr,
            char *typ,char *soinf);

